@@ -24,7 +24,8 @@
 #include <stdio.h>
 #endif
 
-#define LIBCFS_DEBUG
+/* Controlled via configure key */
+/* #define LIBCFS_DEBUG */
 
 #ifndef offsetof
 # define offsetof(typ,memb)     ((unsigned long)((char *)&(((typ *)0)->memb)))
@@ -43,7 +44,13 @@
 #define container_of0(ptr, type, member)                        \
 ({                                                              \
         typeof(ptr) __ptr = (ptr);                              \
-        __ptr ? container_of(__ptr, type, member) : NULL;       \
+        type       *__res;                                      \
+                                                                \
+        if (unlikely(IS_ERR(__ptr) || __ptr == NULL))           \
+                __res = (type *)__ptr;                          \
+        else                                                    \
+                __res = container_of(__ptr, type, member);      \
+        __res;                                                  \
 })
 
 /*
@@ -67,6 +74,9 @@ extern unsigned int libcfs_stack;
 extern unsigned int libcfs_debug;
 extern unsigned int libcfs_printk;
 extern unsigned int libcfs_console_ratelimit;
+extern cfs_duration_t libcfs_console_max_delay;
+extern cfs_duration_t libcfs_console_min_delay;
+extern unsigned int libcfs_console_backoff;
 extern unsigned int libcfs_debug_binary;
 extern char debug_file_path[1024];
 
@@ -75,6 +85,7 @@ int libcfs_debug_str2mask(int *mask, const char *str, int is_subsys);
 
 /* Has there been an LBUG? */
 extern unsigned int libcfs_catastrophe;
+extern unsigned int libcfs_panic_on_lbug;
 
 /*
  * struct ptldebug_header is defined in libcfs/<os>/libcfs.h
@@ -156,18 +167,21 @@ extern unsigned int libcfs_catastrophe;
 # define DEBUG_SUBSYSTEM S_UNDEFINED
 #endif
 
-#define CDEBUG_MAX_LIMIT 600
+#define CDEBUG_DEFAULT_MAX_DELAY (cfs_time_seconds(600))         /* jiffies */
+#define CDEBUG_DEFAULT_MIN_DELAY ((cfs_time_seconds(1) + 1) / 2) /* jiffies */
+#define CDEBUG_DEFAULT_BACKOFF   2
 typedef struct {
         cfs_time_t      cdls_next;
         int             cdls_count;
         cfs_duration_t  cdls_delay;
 } cfs_debug_limit_state_t;
 
-#define CDEBUG_ENABLED (1)
+/* Controlled via configure key */
+/* #define CDEBUG_ENABLED */
 
-#ifdef __KERNEL__
+#if defined(__KERNEL__) || (defined(__arch_lib__) && !defined(LUSTRE_UTILS))
 
-#if CDEBUG_ENABLED
+#ifdef CDEBUG_ENABLED
 #define __CDEBUG(cdls, mask, format, a...)                              \
 do {                                                                    \
         CHECK_STACK();                                                  \
@@ -192,22 +206,9 @@ do {                                            \
 #else /* CDEBUG_ENABLED */
 #define CDEBUG(mask, format, a...) (void)(0)
 #define CDEBUG_LIMIT(mask, format, a...) (void)(0)
+#warning "CDEBUG IS DISABLED. THIS SHOULD NEVER BE DONE FOR PRODUCTION!"
 #endif
 
-#elif defined(__arch_lib__) && !defined(LUSTRE_UTILS)
-
-#define CDEBUG(mask, format, a...)                                      \
-do {                                                                    \
-        if (((mask) & D_CANTMASK) != 0 ||                               \
-            ((libcfs_debug & (mask)) != 0 &&                            \
-             (libcfs_subsystem_debug & DEBUG_SUBSYSTEM) != 0))          \
-                libcfs_debug_msg(NULL, DEBUG_SUBSYSTEM, mask,           \
-                                 __FILE__, __FUNCTION__, __LINE__,      \
-                                 format, ## a);                         \
-} while (0)
-
-#define CDEBUG_LIMIT CDEBUG
-
 #else
 
 #define CDEBUG(mask, format, a...)                                      \
@@ -221,6 +222,13 @@ do {                                                                    \
 
 #endif /* !__KERNEL__ */
 
+/*
+ * Lustre Error Checksum: calculates checksum
+ * of Hex number by XORing each bit.
+ */
+#define LERRCHKSUM(hexnum) (((hexnum) & 0xf) ^ ((hexnum) >> 4 & 0xf) ^ \
+                           ((hexnum) >> 8 & 0xf))
+
 #define CWARN(format, a...)          CDEBUG_LIMIT(D_WARNING, format, ## a)
 #define CERROR(format, a...)         CDEBUG_LIMIT(D_ERROR, format, ## a)
 #define CEMERG(format, a...)         CDEBUG_LIMIT(D_EMERG, format, ## a)
@@ -228,10 +236,13 @@ do {                                                                    \
 #define LCONSOLE(mask, format, a...) CDEBUG(D_CONSOLE | (mask), format, ## a)
 #define LCONSOLE_INFO(format, a...)  CDEBUG_LIMIT(D_CONSOLE, format, ## a)
 #define LCONSOLE_WARN(format, a...)  CDEBUG_LIMIT(D_CONSOLE | D_WARNING, format, ## a)
-#define LCONSOLE_ERROR(format, a...) CDEBUG_LIMIT(D_CONSOLE | D_ERROR, format, ## a)
+#define LCONSOLE_ERROR_MSG(errnum, format, a...) CDEBUG_LIMIT(D_CONSOLE | D_ERROR, \
+                           "%x-%x: " format, errnum, LERRCHKSUM(errnum),  ## a)
+#define LCONSOLE_ERROR(format, a...) LCONSOLE_ERROR_MSG(0x00, format, ## a)
+
 #define LCONSOLE_EMERG(format, a...) CDEBUG(D_CONSOLE | D_EMERG, format, ## a)
 
-#if CDEBUG_ENABLED
+#ifdef CDEBUG_ENABLED
 
 #define GOTO(label, rc)                                                 \
 do {                                                                    \
@@ -245,8 +256,10 @@ do {                                                                    \
 #define GOTO(label, rc) do { ((void)(rc)); goto label; } while (0)
 #endif
 
-#define CDEBUG_ENTRY_EXIT (1)
-#if CDEBUG_ENTRY_EXIT
+/* Controlled via configure key */
+/* #define CDEBUG_ENTRY_EXIT */
+
+#ifdef CDEBUG_ENTRY_EXIT
 
 /*
  * if rc == NULL, we need to code as RETURN((void *)NULL), otherwise
@@ -313,9 +326,6 @@ int libcfs_register_ioctl(struct libcfs_ioctl_handler *hand);
 int libcfs_deregister_ioctl(struct libcfs_ioctl_handler *hand);
 
 /* libcfs tcpip */
-#define LNET_ACCEPTOR_MIN_RESERVED_PORT    512
-#define LNET_ACCEPTOR_MAX_RESERVED_PORT    1023
-
 int libcfs_ipif_query(char *name, int *up, __u32 *ip, __u32 *mask);
 int libcfs_ipif_enumerate(char ***names);
 void libcfs_ipif_free_enumeration(char **names, int n);
@@ -342,6 +352,7 @@ struct lc_watchdog *lc_watchdog_add(int time,
                                     void *data);
 
 /* Enables a watchdog and resets its timer. */
+void lc_watchdog_touch_ms(struct lc_watchdog *lcw, int timeout_ms);
 void lc_watchdog_touch(struct lc_watchdog *lcw);
 
 /* Disable a watchdog; touch it to restart it. */
@@ -356,6 +367,10 @@ void lc_watchdog_dumplog(pid_t pid, void *data);
 /* __KERNEL__ */
 #endif
 
+/* need both kernel and user-land acceptor */
+#define LNET_ACCEPTOR_MIN_RESERVED_PORT    512
+#define LNET_ACCEPTOR_MAX_RESERVED_PORT    1023
+
 /*
  * libcfs pseudo device operations
  *
@@ -461,34 +476,35 @@ struct libcfs_debug_msg_data {
 
 extern int libcfs_debug_vmsg2(cfs_debug_limit_state_t *cdls,
                               int subsys, int mask,
-                              const char *file, const char *fn, const int line, 
-                              const char *format1, va_list args, 
-                              const char *format2, ...);
+                              const char *file, const char *fn, const int line,
+                              const char *format1, va_list args,
+                              const char *format2, ...)
+        __attribute__ ((format (printf, 9, 10)));
 
-#define libcfs_debug_vmsg(cdls, subsys, mask, file, fn, line, format, args) \
-    libcfs_debug_vmsg2(cdls, subsys, mask, file, fn, line, format, args, NULL, NULL)
+#define libcfs_debug_vmsg(cdls, subsys, mask, file, fn, line, format, args)   \
+    libcfs_debug_vmsg2(cdls, subsys, mask, file, fn,line,format,args,NULL,NULL)
 
-#define libcfs_debug_msg(cdls, subsys, mask, file, fn, line, format, a...) \
-    libcfs_debug_vmsg2(cdls, subsys, mask, file, fn, line, NULL, NULL, format, ##a) 
+#define libcfs_debug_msg(cdls, subsys, mask, file, fn, line, format, a...)    \
+    libcfs_debug_vmsg2(cdls, subsys, mask, file, fn,line,NULL,NULL,format, ##a)
 
-#define cdebug_va(cdls, mask, file, func, line, fmt, args)      do {            \
-        CHECK_STACK();                                                          \
-                                                                                \
-        if (((mask) & D_CANTMASK) != 0 ||                                       \
-            ((libcfs_debug & (mask)) != 0 &&                                    \
-             (libcfs_subsystem_debug & DEBUG_SUBSYSTEM) != 0))                  \
-                libcfs_debug_vmsg(cdls, DEBUG_SUBSYSTEM, (mask),                \
-                                  (file), (func), (line), fmt, args);           \
+#define cdebug_va(cdls, mask, file, func, line, fmt, args)      do {          \
+        CHECK_STACK();                                                        \
+                                                                              \
+        if (((mask) & D_CANTMASK) != 0 ||                                     \
+            ((libcfs_debug & (mask)) != 0 &&                                  \
+             (libcfs_subsystem_debug & DEBUG_SUBSYSTEM) != 0))                \
+                libcfs_debug_vmsg(cdls, DEBUG_SUBSYSTEM, (mask),              \
+                                  (file), (func), (line), fmt, args);         \
 } while(0);
 
-#define cdebug(cdls, mask, file, func, line, fmt, a...) do {                    \
-        CHECK_STACK();                                                          \
-                                                                                \
-        if (((mask) & D_CANTMASK) != 0 ||                                       \
-            ((libcfs_debug & (mask)) != 0 &&                                    \
-             (libcfs_subsystem_debug & DEBUG_SUBSYSTEM) != 0))                  \
-                libcfs_debug_msg(cdls, DEBUG_SUBSYSTEM, (mask),                 \
-                                 (file), (func), (line), fmt, ## a);            \
+#define cdebug(cdls, mask, file, func, line, fmt, a...) do {                  \
+        CHECK_STACK();                                                        \
+                                                                              \
+        if (((mask) & D_CANTMASK) != 0 ||                                     \
+            ((libcfs_debug & (mask)) != 0 &&                                  \
+             (libcfs_subsystem_debug & DEBUG_SUBSYSTEM) != 0))                \
+                libcfs_debug_msg(cdls, DEBUG_SUBSYSTEM, (mask),               \
+                                 (file), (func), (line), fmt, ## a);          \
 } while(0);
 
 extern void libcfs_assertion_failed(const char *expr, const char *file,
@@ -499,7 +515,7 @@ static inline void cfs_slow_warning(cfs_time_t now, int seconds, char *msg)
         if (cfs_time_after(cfs_time_current(),
                            cfs_time_add(now, cfs_time_seconds(15))))
                 CERROR("slow %s %lu sec\n", msg,
-                       cfs_duration_sec(cfs_time_sub(cfs_time_current(), now)));
+                       cfs_duration_sec(cfs_time_sub(cfs_time_current(),now)));
 }
 
 /*
@@ -529,18 +545,18 @@ static inline cfs_duration_t cfs_timeout_cap(cfs_duration_t timeout)
  */
 enum cfs_alloc_flags {
         /* allocation is not allowed to block */
-        CFS_ALLOC_ATOMIC = (1 << 0),
+        CFS_ALLOC_ATOMIC = 0x1,
         /* allocation is allowed to block */
-        CFS_ALLOC_WAIT = (1 << 1),
+        CFS_ALLOC_WAIT   = 0x2,
         /* allocation should return zeroed memory */
-        CFS_ALLOC_ZERO   = (1 << 2),
+        CFS_ALLOC_ZERO   = 0x4,
         /* allocation is allowed to call file-system code to free/clean
          * memory */
-        CFS_ALLOC_FS     = (1 << 3),
+        CFS_ALLOC_FS     = 0x8,
         /* allocation is allowed to do io to free/clean memory */
-        CFS_ALLOC_IO     = (1 << 4),
+        CFS_ALLOC_IO     = 0x10,
         /* don't report allocation failure to the console */
-        CFS_ALLOC_NOWARN = (1 << 5),
+        CFS_ALLOC_NOWARN = 0x20,
         /* standard allocator flag combination */
         CFS_ALLOC_STD    = CFS_ALLOC_FS | CFS_ALLOC_IO,
         CFS_ALLOC_USER   = CFS_ALLOC_WAIT | CFS_ALLOC_FS | CFS_ALLOC_IO,
@@ -550,40 +566,10 @@ enum cfs_alloc_flags {
 enum cfs_alloc_page_flags {
         /* allow to return page beyond KVM. It has to be mapped into KVM by
          * cfs_page_map(); */
-        CFS_ALLOC_HIGH   = (1 << 5),
+        CFS_ALLOC_HIGH   = 0x40,
         CFS_ALLOC_HIGHUSER = CFS_ALLOC_WAIT | CFS_ALLOC_FS | CFS_ALLOC_IO | CFS_ALLOC_HIGH,
 };
 
-/*
- * portable UNIX device file identification. (This is not _very_
- * portable. Probably makes no sense for Windows.)
- */
-/*
- * Platform defines
- *
- * cfs_rdev_t
- */
-
-typedef unsigned int cfs_major_nr_t;
-typedef unsigned int cfs_minor_nr_t;
-
-/*
- * Defined by platform.
- */
-cfs_rdev_t     cfs_rdev_build(cfs_major_nr_t major, cfs_minor_nr_t minor);
-cfs_major_nr_t cfs_rdev_major(cfs_rdev_t rdev);
-cfs_minor_nr_t cfs_rdev_minor(cfs_rdev_t rdev);
-
-/*
- * Generic on-wire rdev format.
- */
-
-typedef __u32 cfs_wire_rdev_t;
-
-cfs_wire_rdev_t cfs_wire_rdev_build(cfs_major_nr_t major, cfs_minor_nr_t minor);
-cfs_major_nr_t  cfs_wire_rdev_major(cfs_wire_rdev_t rdev);
-cfs_minor_nr_t  cfs_wire_rdev_minor(cfs_wire_rdev_t rdev);
-
 /*
  * Drop into debugger, if possible. Implementation is provided by platform.
  */
