@@ -21,6 +21,7 @@
 #include <lustre_import.h>
 #include <lustre_handles.h>
 #include <lustre_export.h> /* for obd_export, for LDLM_DEBUG */
+#include <interval_tree.h> /* for interval_node{}, ldlm_extent */
 
 struct obd_ops;
 struct obd_device;
@@ -43,8 +44,8 @@ typedef enum {
 } ldlm_error_t;
 
 typedef enum {
-        LDLM_NAMESPACE_SERVER = 0,
-        LDLM_NAMESPACE_CLIENT = 1
+        LDLM_NAMESPACE_SERVER = 1 << 0,
+        LDLM_NAMESPACE_CLIENT = 1 << 1
 } ldlm_side_t;
 
 #define LDLM_FL_LOCK_CHANGED   0x000001 /* extent, mode, or resource changed */
@@ -217,15 +218,11 @@ struct ldlm_lock;
 struct ldlm_resource;
 struct ldlm_namespace;
 
-typedef int (*ldlm_pool_recalc_t)(struct ldlm_pool *pl);
-                                  
-typedef int (*ldlm_pool_shrink_t)(struct ldlm_pool *pl,
-                                  int nr, unsigned int gfp_mask);
-
-enum {
-        LDLM_POOL_CTL_RECALC = 1 << 0, /* Pool recalc is enabled */
-        LDLM_POOL_CTL_SHRINK = 1 << 1, /* Pool shrink is enabled */
-        LDLM_POOL_CTL_FULL   = (LDLM_POOL_CTL_RECALC | LDLM_POOL_CTL_SHRINK)
+struct ldlm_pool_ops {
+        int (*po_recalc)(struct ldlm_pool *pl);
+        int (*po_shrink)(struct ldlm_pool *pl, int nr, 
+                         unsigned int gfp_mask);
+        int (*po_setup)(struct ldlm_pool *pl, int limit);
 };
 
 /* One second for pools thread check interval. */
@@ -235,51 +232,39 @@ enum {
 #define LDLM_POOLS_MODEST_MARGIN (5)
 
 /* A change to SLV in % after which we want to wake up pools thread asap. */
-#define LDLM_POOLS_FAST_SLV_CHANGE (5)
+#define LDLM_POOLS_FAST_SLV_CHANGE (50)
 
 struct ldlm_pool {
         /* Common pool fields */
-        cfs_proc_dir_entry_t  *pl_proc_dir;           /* Pool proc directory. */
-        char                   pl_name[100];          /* Pool name, should be long 
-                                                       * enough to contain complex 
-                                                       * proc entry name. */
-        spinlock_t             pl_lock;               /* Lock for protecting slv/clv 
-                                                       * updates. */
-        atomic_t               pl_limit;              /* Number of allowed locks in
-                                                       * in pool, both, client and 
-                                                       * server side. */
-        atomic_t               pl_granted;            /* Number of granted locks. */
-        atomic_t               pl_grant_rate;         /* Grant rate per T. */
-        atomic_t               pl_cancel_rate;        /* Cancel rate per T. */
-        atomic_t               pl_grant_speed;        /* Grant speed (GR - CR) per T. */
-        __u64                  pl_server_lock_volume; /* Server lock volume. Protected 
-                                                       * by pl_lock. */
-        cfs_time_t             pl_update_time;        /* Time when last slv from server 
-                                                       * was obtained. */
-        ldlm_pool_recalc_t     pl_recalc;             /* Recalc callback func pointer. */
-        ldlm_pool_shrink_t     pl_shrink;             /* Shrink callback func pointer. */
-        int                    pl_control;            /* Pool features mask */
-        
-        /* Server side pool fields */
-        atomic_t               pl_grant_plan;         /* Planned number of granted 
-                                                       * locks for next T. */
-        atomic_t               pl_grant_step;         /* Grant plan step for next T. */
-
-        /* Client side pool related fields */
+        cfs_proc_dir_entry_t  *pl_proc_dir;      /* Pool proc directory. */
+        char                   pl_name[100];     /* Pool name, should be long 
+                                                  * enough to contain complex
+                                                  * proc entry name. */
+        spinlock_t             pl_lock;          /* Lock for protecting slv/clv 
+                                                  * updates. */
+        atomic_t               pl_limit;         /* Number of allowed locks in
+                                                  * in pool, both, client and 
+                                                  * server side. */
+        atomic_t               pl_granted;       /* Number of granted locks. */
+        atomic_t               pl_grant_rate;    /* Grant rate per T. */
+        atomic_t               pl_cancel_rate;   /* Cancel rate per T. */
+        atomic_t               pl_grant_speed;   /* Grant speed (GR-CR) per T. */
+        __u64                  pl_server_lock_volume; /* Server lock volume. 
+                                                  * Protected by pl_lock */
         atomic_t               pl_lock_volume_factor; /* Lock volume factor. */
-        struct lprocfs_stats  *pl_stats;              /* Pool statistics. */
+
+        time_t                 pl_recalc_time;   /* Time when last slv from 
+                                                  * server was obtained. */
+        struct ldlm_pool_ops  *pl_ops;           /* Recalc and shrink ops. */ 
+
+        int                    pl_grant_plan;    /* Planned number of granted 
+                                                  * locks for next T. */
+        int                    pl_grant_step;    /* Grant plan step for next 
+                                                  * T. */
+
+        struct lprocfs_stats  *pl_stats;         /* Pool statistics. */
 };
 
-static inline int pool_recalc_enabled(struct ldlm_pool *pl)
-{
-        return pl->pl_control & LDLM_POOL_CTL_RECALC;
-}
-
-static inline int pool_shrink_enabled(struct ldlm_pool *pl)
-{
-        return pl->pl_control & LDLM_POOL_CTL_SHRINK;
-}
-
 typedef int (*ldlm_res_policy)(struct ldlm_namespace *, struct ldlm_lock **,
                                void *req_cookie, ldlm_mode_t mode, int flags,
                                void *data);
@@ -301,12 +286,21 @@ typedef enum {
 #define NS_DEFAULT_CONTENTION_SECONDS 2
 #define NS_DEFAULT_CONTENDED_LOCKS 32
 
+/* Default value for ->ns_shrink_thumb. If lock is not extent one its cost 
+ * is one page. Here we have 256 pages which is 1M on i386. Thus by default
+ * all extent locks which have more than 1M long extent will be kept in lru,
+ * others (including ibits locks) will be canceled on memory pressure event. */
+#define LDLM_LOCK_SHRINK_THUMB 256
+
 struct ldlm_namespace {
         char                  *ns_name;
         ldlm_side_t            ns_client; /* is this a client-side lock tree? */
-        __u64                  ns_connect_flags; /* client side connect flags 
-                                                  * supported by server */
-        struct list_head      *ns_hash; /* hash table for ns */
+        __u64                  ns_connect_flags; /* ns connect flags supported
+                                           * by server (may be changed via proc,
+                                           * lru resize may be disabled/enabled) */
+        __u64                  ns_orig_connect_flags; /* client side orig connect
+                                           * flags supported by server */
+        struct list_head      *ns_hash;   /* hash table for ns */
         spinlock_t             ns_hash_lock;
         __u32                  ns_refcount; /* count of resources in the hash */
         struct list_head       ns_root_list; /* all root resources in ns */
@@ -318,6 +312,9 @@ struct ldlm_namespace {
 
         unsigned int           ns_max_unused;
         unsigned int           ns_max_age;
+        
+        /* Lower limit to number of pages in lock to keep it in cache */
+        unsigned int           ns_shrink_thumb;
         cfs_time_t             ns_next_dump;   /* next debug dump, jiffies */
 
         atomic_t               ns_locks;
@@ -337,12 +334,34 @@ struct ldlm_namespace {
         unsigned               ns_contention_time;
         /* limit size of nolock requests, in bytes */
         unsigned               ns_max_nolock_size;
+
+        struct adaptive_timeout ns_at_estimate;/* estimated lock callback time*/
 };
 
+static inline int ns_is_client(struct ldlm_namespace *ns)
+{
+        LASSERT(ns != NULL);
+        LASSERT(!(ns->ns_client & ~(LDLM_NAMESPACE_CLIENT | 
+                                    LDLM_NAMESPACE_SERVER)));
+        LASSERT(ns->ns_client == LDLM_NAMESPACE_CLIENT ||
+                ns->ns_client == LDLM_NAMESPACE_SERVER);
+        return ns->ns_client == LDLM_NAMESPACE_CLIENT;
+}
+
+static inline int ns_is_server(struct ldlm_namespace *ns)
+{
+        LASSERT(ns != NULL);
+        LASSERT(!(ns->ns_client & ~(LDLM_NAMESPACE_CLIENT | 
+                                    LDLM_NAMESPACE_SERVER)));
+        LASSERT(ns->ns_client == LDLM_NAMESPACE_CLIENT ||
+                ns->ns_client == LDLM_NAMESPACE_SERVER);
+        return ns->ns_client == LDLM_NAMESPACE_SERVER;
+}
+
 static inline int ns_connect_lru_resize(struct ldlm_namespace *ns)
 {
         LASSERT(ns != NULL);
-        return ns->ns_connect_flags & OBD_CONNECT_LRU_RESIZE;
+        return !!(ns->ns_connect_flags & OBD_CONNECT_LRU_RESIZE);
 }
 
 /*
@@ -364,6 +383,23 @@ typedef int (*ldlm_completion_callback)(struct ldlm_lock *lock, int flags,
                                         void *data);
 typedef int (*ldlm_glimpse_callback)(struct ldlm_lock *lock, void *data);
 
+/* Interval node data for each LDLM_EXTENT lock */
+struct ldlm_interval {
+        struct interval_node li_node;   /* node for tree mgmt */
+        struct list_head     li_group;  /* the locks which have the same 
+                                         * policy - group of the policy */
+};
+#define to_ldlm_interval(n) container_of(n, struct ldlm_interval, li_node)
+
+/* the interval tree must be accessed inside the resource lock. */
+struct ldlm_interval_tree {
+        /* tree size, this variable is used to count
+         * granted PW locks in ldlm_extent_policy()*/
+        int                   lit_size;
+        ldlm_mode_t           lit_mode; /* lock mode */
+        struct interval_node *lit_root; /* actually ldlm_interval */
+};
+
 struct ldlm_lock {
         struct portals_handle l_handle; // must be first in the structure
         atomic_t              l_refc;
@@ -384,6 +420,8 @@ struct ldlm_lock {
         struct list_head      l_sl_mode;        // skip pointer for request mode
         struct list_head      l_sl_policy;      // skip pointer for inodebits
 
+        struct ldlm_interval *l_tree_node;      /* tree node for ldlm_extent */
+
         /* protected by led_lock */
         struct list_head      l_export_chain; // per-export chain of locks
 
@@ -421,6 +459,10 @@ struct ldlm_lock {
         void                 *l_lvb_data;       /* an LVB received during */
         void                 *l_lvb_swabber;    /* an enqueue */
         void                 *l_ast_data;
+        spinlock_t            l_extents_list_lock;
+        struct list_head      l_extents_list;
+
+        struct list_head      l_cache_locks_list;
 
         /* Server-side-only members */
 
@@ -457,6 +499,8 @@ struct ldlm_resource {
         struct ldlm_res_id     lr_name;
         atomic_t               lr_refcount;
 
+        struct ldlm_interval_tree lr_itree[LCK_MODE_NUM];  /* interval trees*/
+
         /* Server-side-only lock value block elements */
         struct semaphore       lr_lvb_sem;
         __u32                  lr_lvb_len;
@@ -572,8 +616,8 @@ int ldlm_request_cancel(struct ptlrpc_request *req,
                         struct ldlm_request *dlm_req, int first);
 int ldlm_del_waiting_lock(struct ldlm_lock *lock);
 int ldlm_refresh_waiting_lock(struct ldlm_lock *lock);
-int ldlm_get_ref(ldlm_side_t client);
-void ldlm_put_ref(ldlm_side_t client, int force);
+int ldlm_get_ref(void);
+void ldlm_put_ref(int force);
 
 /* ldlm_lock.c */
 ldlm_processing_policy ldlm_get_processing_policy(struct ldlm_resource *res);
@@ -637,11 +681,12 @@ void ldlm_lock_addref(struct lustre_handle *lockh, __u32 mode);
 void ldlm_lock_decref(struct lustre_handle *lockh, __u32 mode);
 void ldlm_lock_decref_and_cancel(struct lustre_handle *lockh, __u32 mode);
 void ldlm_lock_allow_match(struct ldlm_lock *lock);
-int ldlm_lock_match(struct ldlm_namespace *ns, int flags, struct ldlm_res_id *,
-                    ldlm_type_t type, ldlm_policy_data_t *, ldlm_mode_t mode,
-                    struct lustre_handle *);
+ldlm_mode_t ldlm_lock_match(struct ldlm_namespace *ns, int flags,
+                            struct ldlm_res_id *, ldlm_type_t type,
+                            ldlm_policy_data_t *, ldlm_mode_t mode,
+                            struct lustre_handle *);
 struct ldlm_resource *ldlm_lock_convert(struct ldlm_lock *lock, int new_mode,
-                                        int *flags);
+                                        __u32 *flags);
 void ldlm_lock_cancel(struct ldlm_lock *lock);
 void ldlm_cancel_locks_for_export(struct obd_export *export);
 void ldlm_reprocess_all(struct ldlm_resource *res);
@@ -699,12 +744,17 @@ int ldlm_cli_enqueue(struct obd_export *exp, struct ptlrpc_request **req,
 struct ptlrpc_request *ldlm_prep_enqueue_req(struct obd_export *exp,
                                              int bufcount, int *size,
                                              struct list_head *head, int count);
+struct ptlrpc_request *ldlm_prep_elc_req(struct obd_export *exp, int version,
+                                         int opc, int bufcount, int *size,
+                                         int bufoff, int canceloff,
+                                         struct list_head *cancels, int count);
 int ldlm_cli_enqueue_fini(struct obd_export *exp, struct ptlrpc_request *req,
                           ldlm_type_t type, __u8 with_policy, ldlm_mode_t mode,
                           int *flags, void *lvb, __u32 lvb_len,
                           void *lvb_swabber, struct lustre_handle *lockh,
                           int rc);
-int ldlm_cli_enqueue_local(struct ldlm_namespace *ns, struct ldlm_res_id res_id,
+int ldlm_cli_enqueue_local(struct ldlm_namespace *ns,
+                           struct ldlm_res_id *res_id,
                            ldlm_type_t type, ldlm_policy_data_t *policy,
                            ldlm_mode_t mode, int *flags,
                            ldlm_blocking_callback blocking,
@@ -714,7 +764,7 @@ int ldlm_cli_enqueue_local(struct ldlm_namespace *ns, struct ldlm_res_id res_id,
                            struct lustre_handle *lockh);
 int ldlm_server_ast(struct lustre_handle *lockh, struct ldlm_lock_desc *new,
                     void *data, __u32 data_len);
-int ldlm_cli_convert(struct lustre_handle *, int new_mode, int *flags);
+int ldlm_cli_convert(struct lustre_handle *, int new_mode, __u32 *flags);
 int ldlm_cli_update_pool(struct ptlrpc_request *req);
 int ldlm_cli_cancel(struct lustre_handle *lockh);
 int ldlm_cli_cancel_unused(struct ldlm_namespace *, struct ldlm_res_id *,
@@ -725,10 +775,9 @@ int ldlm_cli_join_lru(struct ldlm_namespace *, struct ldlm_res_id *, int join);
 int ldlm_cancel_resource_local(struct ldlm_resource *res,
                                struct list_head *cancels,
                                ldlm_policy_data_t *policy, ldlm_mode_t mode,
-                               int lock_flags, int flags, void *opaque);
+                               int lock_flags, int cancel_flags, void *opaque);
 int ldlm_cli_cancel_list(struct list_head *head, int count,
                          struct ptlrpc_request *req, int off);
-
 /* mds/handler.c */
 /* This has to be here because recursive inclusion sucks. */
 int intent_disposition(struct ldlm_reply *rep, int flag);
@@ -764,8 +813,8 @@ struct ldlm_resource * lock_res_and_lock(struct ldlm_lock *lock);
 void unlock_res_and_lock(struct ldlm_lock *lock);
 
 /* ldlm_pool.c */
-int ldlm_pools_init(ldlm_side_t client);
 void ldlm_pools_recalc(ldlm_side_t client);
+int ldlm_pools_init(void);
 void ldlm_pools_fini(void);
 void ldlm_pools_wakeup(void);
 
@@ -774,7 +823,7 @@ int ldlm_pool_init(struct ldlm_pool *pl, struct ldlm_namespace *ns,
 int ldlm_pool_shrink(struct ldlm_pool *pl, int nr, 
                      unsigned int gfp_mask);
 void ldlm_pool_fini(struct ldlm_pool *pl);
-int ldlm_pool_setup(struct ldlm_pool *pl, __u32 limit);
+int ldlm_pool_setup(struct ldlm_pool *pl, int limit);
 int ldlm_pool_recalc(struct ldlm_pool *pl);
 __u64 ldlm_pool_get_slv(struct ldlm_pool *pl);
 __u32 ldlm_pool_get_limit(struct ldlm_pool *pl);
