@@ -190,16 +190,11 @@ static void mdc_realloc_openmsg(struct ptlrpc_request *req,
         struct lustre_msg *new_msg;
 
         old_len = lustre_msg_buflen(old_msg, DLM_INTENT_REC_OFF + 2);
-        /* save old size */
-        old_size = lustre_msg_size(lustre_request_magic(req),
-                                   req->rq_reqmsg->lm_bufcount,
-                                   req->rq_reqmsg->lm_buflens);
-
+        old_size = lustre_packed_msg_size(old_msg);
         lustre_msg_set_buflen(old_msg, DLM_INTENT_REC_OFF + 2,
                               body->eadatasize);
-        new_size = lustre_msg_size(lustre_request_magic(req),
-                                   req->rq_reqmsg->lm_bufcount,
-                                   req->rq_reqmsg->lm_buflens);
+        new_size = lustre_packed_msg_size(old_msg);
+
         OBD_ALLOC(new_msg, new_size);
         if (new_msg != NULL) {
                 DEBUG_REQ(D_INFO, req, "replace reqmsg for larger EA %u\n",
@@ -247,49 +242,53 @@ static struct ptlrpc_request *mdc_intent_open_pack(struct obd_export *exp,
                            [DLM_REPLY_REC_OFF+1] = obddev->u.cli.
                                                         cl_max_mds_easize,
                            [DLM_REPLY_REC_OFF+2] = LUSTRE_POSIX_ACL_MAX_SIZE };
-                CFS_LIST_HEAD(cancels);
-                int count = 0;
-                int mode;
+        CFS_LIST_HEAD(cancels);
+        int do_join = (it->it_flags & O_JOIN_FILE) && data->data;
+        int count = 0;
+        int mode;
         int rc;
-                
-                it->it_create_mode |= S_IFREG;
+
+        it->it_create_mode |= S_IFREG;
 
         rc = lustre_msg_size(class_exp2cliimp(exp)->imp_msg_magic, 6, size);
-                if (rc & (rc - 1))
-                        size[DLM_INTENT_REC_OFF + 2] =
+        if (rc & (rc - 1))
+                size[DLM_INTENT_REC_OFF + 2] =
                          min(size[DLM_INTENT_REC_OFF + 2] + round_up(rc) - rc,
                                      obddev->u.cli.cl_max_mds_easize);
 
                 /* If inode is known, cancel conflicting OPEN locks. */
         if (data->fid2.id) {
-                        if (it->it_flags & (FMODE_WRITE|MDS_OPEN_TRUNC))
-                                mode = LCK_CW;
+                if (it->it_flags & (FMODE_WRITE|MDS_OPEN_TRUNC))
+                        mode = LCK_CW;
 #ifdef FMODE_EXEC
-                        else if (it->it_flags & FMODE_EXEC)
-                                mode = LCK_PR;
+                else if (it->it_flags & FMODE_EXEC)
+                        mode = LCK_PR;
 #endif
-                        else 
-                                mode = LCK_CR;
-                count = mdc_resource_get_unused(exp, &data->fid2, &cancels,
-                                                mode, MDS_INODELOCK_OPEN);
-                }
-
-                /* If CREATE or JOIN_FILE, cancel parent's UPDATE lock. */
-                if (it->it_op & IT_CREAT || it->it_flags & O_JOIN_FILE)
-                        mode = LCK_EX;
                 else
                         mode = LCK_CR;
+                count = mdc_resource_get_unused(exp, &data->fid2, &cancels,
+                                                mode, MDS_INODELOCK_OPEN);
+        }
+
+        /* If CREATE or JOIN_FILE, cancel parent's UPDATE lock. */
+        if (it->it_op & IT_CREAT || do_join)
+                mode = LCK_EX;
+        else
+                mode = LCK_CR;
         count += mdc_resource_get_unused(exp, &data->fid1, &cancels, mode,
                                          MDS_INODELOCK_UPDATE);
-                if (it->it_flags & O_JOIN_FILE) {
+        if (do_join) {
                 __u64 head_size = (*(__u64 *)data->data);
                         /* join is like an unlink of the tail */
                 size[DLM_INTENT_REC_OFF + 3] = sizeof(struct mds_rec_join);
                 req = ldlm_prep_enqueue_req(exp, 7, size, &cancels, count);
-                mdc_join_pack(req, DLM_INTENT_REC_OFF + 3, data, head_size);
-                } else {
+                if (req)
+                        mdc_join_pack(req, DLM_INTENT_REC_OFF + 3, data, 
+                                      head_size);
+        } else {
                 req = ldlm_prep_enqueue_req(exp, 6, size, &cancels, count);
-                }
+                it->it_flags &= ~O_JOIN_FILE;
+        }
 
         if (req) {
                 spin_lock(&req->rq_lock);
@@ -407,6 +406,7 @@ static int mdc_finish_enqueue(struct obd_export *exp,
         struct ldlm_reply *lockrep;
         ENTRY;
 
+        LASSERT(rc >= 0);
         /* Similarly, if we're going to replay this request, we don't want to
          * actually get a lock, just perform the intent. */
         if (req->rq_transno || req->rq_replay) {
@@ -419,12 +419,6 @@ static int mdc_finish_enqueue(struct obd_export *exp,
                 einfo->ei_mode = 0;
                 memset(lockh, 0, sizeof(*lockh));
                 rc = 0;
-        } else if (rc != 0) {
-                CERROR("ldlm_cli_enqueue: %d\n", rc);
-                LASSERTF(rc < 0, "rc %d\n", rc);
-                mdc_clear_replay_flag(req, rc);
-                ptlrpc_req_finished(req);
-                RETURN(rc);
         } else { /* rc = 0 */
                 struct ldlm_lock *lock = ldlm_handle2lock(lockh);
                 LASSERT(lock);
@@ -442,7 +436,8 @@ static int mdc_finish_enqueue(struct obd_export *exp,
         lockrep = lustre_msg_buf(req->rq_repmsg, DLM_LOCKREPLY_OFF,
                                  sizeof(*lockrep));
         LASSERT(lockrep != NULL);                 /* checked by ldlm_cli_enqueue() */
-        LASSERT_REPSWABBED(req, DLM_LOCKREPLY_OFF); /* swabbed by ldlm_cli_enqueue() */
+        /* swabbed by ldlm_cli_enqueue() */
+        LASSERT(lustre_rep_swabbed(req, DLM_LOCKREPLY_OFF));
 
         it->d.lustre.it_disposition = (int)lockrep->lock_policy_res1;
         it->d.lustre.it_status = (int)lockrep->lock_policy_res2;
@@ -587,53 +582,45 @@ int mdc_enqueue(struct obd_export *exp, struct ldlm_enqueue_info *einfo,
                               0, NULL, lockh, 0);
         mdc_exit_request(&obddev->u.cli);
         mdc_put_rpc_lock(obddev->u.cli.cl_rpc_lock, it);
-
+        if (rc < 0) {
+                CERROR("ldlm_cli_enqueue: %d\n", rc);
+                mdc_clear_replay_flag(req, rc);
+                ptlrpc_req_finished(req);
+                RETURN(rc);
+        }
         rc = mdc_finish_enqueue(exp, req, einfo, it, lockh, rc);
 
         RETURN(rc);
 }
 EXPORT_SYMBOL(mdc_enqueue);
 
-int mdc_revalidate_lock(struct obd_export *exp,
-                        struct lookup_intent *it,
+int mdc_revalidate_lock(struct obd_export *exp, struct lookup_intent *it,
                         struct ll_fid *fid)
 {
                 /* We could just return 1 immediately, but since we should only
                  * be called in revalidate_it if we already have a lock, let's
                  * verify that. */
         struct ldlm_res_id res_id = {.name ={fid->id, fid->generation}};
-                struct lustre_handle lockh;
-                ldlm_policy_data_t policy;
-                int mode = LCK_CR;
-        int rc;
+        struct lustre_handle lockh;
+        ldlm_policy_data_t policy;
+        ldlm_mode_t mode;
 
-                /* As not all attributes are kept under update lock, e.g. 
-                   owner/group/acls are under lookup lock, we need both 
-                   ibits for GETATTR. */
-                policy.l_inodebits.bits = (it->it_op == IT_GETATTR) ?
-                        MDS_INODELOCK_UPDATE | MDS_INODELOCK_LOOKUP :
-                        MDS_INODELOCK_LOOKUP;
+        /* As not all attributes are kept under update lock, e.g. 
+           owner/group/acls are under lookup lock, we need both 
+           ibits for GETATTR. */
+        policy.l_inodebits.bits = (it->it_op == IT_GETATTR) ?
+                MDS_INODELOCK_UPDATE | MDS_INODELOCK_LOOKUP :
+                MDS_INODELOCK_LOOKUP;
 
-        rc = ldlm_lock_match(exp->exp_obd->obd_namespace, LDLM_FL_BLOCK_GRANTED,
-                             &res_id, LDLM_IBITS, &policy, LCK_CR, &lockh);
-                if (!rc) {
-                        mode = LCK_CW;
-                        rc = ldlm_lock_match(exp->exp_obd->obd_namespace,
-                                     LDLM_FL_BLOCK_GRANTED, &res_id, LDLM_IBITS,
-                                     &policy, LCK_CW, &lockh);
-                }
-                if (!rc) {
-                        mode = LCK_PR;
-                        rc = ldlm_lock_match(exp->exp_obd->obd_namespace,
-                                     LDLM_FL_BLOCK_GRANTED, &res_id, LDLM_IBITS,
-                                     &policy, LCK_PR, &lockh);
-                }
-                if (rc) {
+        mode = ldlm_lock_match(exp->exp_obd->obd_namespace,
+                               LDLM_FL_BLOCK_GRANTED, &res_id, LDLM_IBITS,
+                               &policy, LCK_CR|LCK_CW|LCK_PR|LCK_PW, &lockh);
+        if (mode) {
                 memcpy(&it->d.lustre.it_lock_handle, &lockh, sizeof(lockh));
-                        it->d.lustre.it_lock_mode = mode;
-                }
+                it->d.lustre.it_lock_mode = mode;
+        }
 
-        return rc;
+        return !!mode;
 }
 EXPORT_SYMBOL(mdc_revalidate_lock);
 
@@ -665,8 +652,10 @@ static int mdc_finish_intent_lock(struct obd_export *exp,
 
         mds_body = lustre_msg_buf(req->rq_repmsg, DLM_REPLY_REC_OFF,
                                   sizeof(*mds_body));
-        LASSERT(mds_body != NULL);           /* mdc_enqueue checked */
-        LASSERT_REPSWABBED(req, DLM_REPLY_REC_OFF); /* mdc_enqueue swabbed */
+        /* mdc_enqueue checked */
+        LASSERT(mds_body != NULL);
+        /* mdc_enqueue swabbed */
+        LASSERT(lustre_rep_swabbed(req, DLM_REPLY_REC_OFF));
 
         /* If we were revalidating a fid/name pair, mark the intent in
          * case we fail and get called again from lookup */
@@ -843,9 +832,16 @@ static int mdc_intent_getattr_async_interpret(struct ptlrpc_request *req,
         obddev = class_exp2obd(exp);
 
         mdc_exit_request(&obddev->u.cli);
+        if (OBD_FAIL_CHECK(OBD_FAIL_MDC_GETATTR_ENQUEUE))
+                rc = -ETIMEDOUT;
 
         rc = ldlm_cli_enqueue_fini(exp, req, einfo->ei_type, 1, einfo->ei_mode,
                                    &flags, NULL, 0, NULL, lockh, rc);
+        if (rc < 0) {
+                CERROR("ldlm_cli_enqueue_fini: %d\n", rc);
+                mdc_clear_replay_flag(req, rc);
+                GOTO(out, rc);
+        }
 
         rc = mdc_finish_enqueue(exp, req, einfo, it, lockh, rc);
         if (rc)
