@@ -48,7 +48,6 @@
 #include <linux/stat.h>
 #include <asm/uaccess.h>
 #include <linux/slab.h>
-#include <asm/segment.h>
 
 #include <obd_support.h>
 #include <lustre_lib.h>
@@ -159,7 +158,7 @@ static int mds_setattr_unpack(struct ptlrpc_request *req, int offset,
         LTIME_S(attr->ia_ctime) = rec->sa_ctime;
         r->ur_flags = rec->sa_attr_flags;
 
-        LASSERT_REQSWAB (req, offset + 1);
+        lustre_set_req_swabbed(req, offset + 1);
         r->ur_eadatalen = lustre_msg_buflen(req->rq_reqmsg, offset + 1);
         if (r->ur_eadatalen) {
                 r->ur_eadata = lustre_msg_buf(req->rq_reqmsg, offset + 1, 0);
@@ -205,13 +204,13 @@ static int mds_create_unpack(struct ptlrpc_request *req, int offset,
         r->ur_time = rec->cr_time;
         r->ur_flags = rec->cr_flags;
 
-        LASSERT_REQSWAB(req, offset + 1);
+        lustre_set_req_swabbed(req, offset + 1);
         r->ur_name = lustre_msg_string(req->rq_reqmsg, offset + 1, 0);
         if (r->ur_name == NULL)
                 RETURN (-EFAULT);
         r->ur_namelen = lustre_msg_buflen(req->rq_reqmsg, offset + 1);
 
-        LASSERT_REQSWAB(req, offset + 2);
+        lustre_set_req_swabbed(req, offset + 2);
         r->ur_tgtlen = lustre_msg_buflen(req->rq_reqmsg, offset + 2);
         if (r->ur_tgtlen) {
                 /* NB for now, we only seem to pass NULL terminated symlink
@@ -255,7 +254,7 @@ static int mds_link_unpack(struct ptlrpc_request *req, int offset,
         r->ur_fid2 = &rec->lk_fid2;
         r->ur_time = rec->lk_time;
 
-        LASSERT_REQSWAB(req, offset + 1);
+        lustre_set_req_swabbed(req, offset + 1);
         r->ur_name = lustre_msg_string(req->rq_reqmsg, offset + 1, 0);
         if (r->ur_name == NULL)
                 RETURN (-EFAULT);
@@ -291,7 +290,7 @@ static int mds_unlink_unpack(struct ptlrpc_request *req, int offset,
         r->ur_fid2 = &rec->ul_fid2;
         r->ur_time = rec->ul_time;
 
-        LASSERT_REQSWAB(req, offset + 1);
+        lustre_set_req_swabbed(req, offset + 1);
         r->ur_name = lustre_msg_string(req->rq_reqmsg, offset + 1, 0);
         if (r->ur_name == NULL)
                 RETURN(-EFAULT);
@@ -327,13 +326,13 @@ static int mds_rename_unpack(struct ptlrpc_request *req, int offset,
         r->ur_fid2 = &rec->rn_fid2;
         r->ur_time = rec->rn_time;
 
-        LASSERT_REQSWAB (req, offset + 1);
+        lustre_set_req_swabbed(req, offset + 1);
         r->ur_name = lustre_msg_string(req->rq_reqmsg, offset + 1, 0);
         if (r->ur_name == NULL)
                 RETURN(-EFAULT);
         r->ur_namelen = lustre_msg_buflen(req->rq_reqmsg, offset + 1);
 
-        LASSERT_REQSWAB (req, offset + 2);
+        lustre_set_req_swabbed(req, offset + 2);
         r->ur_tgt = lustre_msg_string(req->rq_reqmsg, offset + 2, 0);
         if (r->ur_tgt == NULL)
                 RETURN(-EFAULT);
@@ -371,13 +370,13 @@ static int mds_open_unpack(struct ptlrpc_request *req, int offset,
         r->ur_time = rec->cr_time;
         r->ur_flags = rec->cr_flags;
 
-        LASSERT_REQSWAB(req, offset + 1);
+        lustre_set_req_swabbed(req, offset + 1);
         r->ur_name = lustre_msg_string(req->rq_reqmsg, offset + 1, 0);
         if (r->ur_name == NULL)
                 RETURN(-EFAULT);
         r->ur_namelen = lustre_msg_buflen(req->rq_reqmsg, offset + 1);
 
-        LASSERT_REQSWAB(req, offset + 2);
+        lustre_set_req_swabbed(req, offset + 2);
         r->ur_eadatalen = lustre_msg_buflen(req->rq_reqmsg, offset + 2);
         if (r->ur_eadatalen) {
                 r->ur_eadata = lustre_msg_buf(req->rq_reqmsg, offset + 2, 0);
@@ -428,6 +427,28 @@ int mds_update_unpack(struct ptlrpc_request *req, int offset,
         RETURN(rc);
 }
 
+void mds_root_squash(struct mds_obd *mds, lnet_nid_t *peernid,
+                     __u32 *fsuid, __u32 *fsgid, __u32 *cap,
+                     __u32 *suppgid, __u32 *suppgid2)
+{
+        if (!mds->mds_squash_uid || *fsuid)
+               return;
+
+        if (*peernid == mds->mds_nosquash_nid)
+                return;
+
+        CDEBUG(D_OTHER, "squash req from %s, (%d:%d/%x)=>(%d:%d/%x)\n",
+               libcfs_nid2str(*peernid), *fsuid, *fsgid, *cap,
+               mds->mds_squash_uid, mds->mds_squash_gid, 0);
+
+        *fsuid = mds->mds_squash_uid;
+        *fsgid = mds->mds_squash_gid;
+        *cap = 0;
+        *suppgid = -1;
+        if (suppgid2)
+                *suppgid2 = -1;
+}
+
 int mds_init_ucred(struct lvfs_ucred *ucred, struct ptlrpc_request *req,
                    int offset)
 {
@@ -446,6 +467,10 @@ int mds_init_ucred(struct lvfs_ucred *ucred, struct ptlrpc_request *req,
         } else
 #endif
         {
+                mds_root_squash(mds, &req->rq_peer.nid, &body->fsuid,
+                                &body->fsgid, &body->capability,
+                                &body->suppgid, NULL);
+
                 ucred->luc_fsuid = body->fsuid;
                 ucred->luc_fsgid = body->fsgid;
                 ucred->luc_cap = body->capability;
