@@ -83,6 +83,15 @@ int lustre_msg_check_version(struct lustre_msg *msg, __u32 version)
         }
 }
 
+/* early reply size */
+int lustre_msg_early_size() {
+        static int size = 0;
+        if (!size) 
+                size = lustre_msg_size(LUSTRE_MSG_MAGIC_V2, 1, NULL);
+        return size;
+}
+EXPORT_SYMBOL(lustre_msg_early_size);
+
 static inline int lustre_msg_size_v1(int count, int *lengths)
 {
         int size;
@@ -109,7 +118,11 @@ static inline int lustre_msg_size_v2(int count, int *lengths)
 }
 
 /* This returns the size of the buffer that is required to hold a lustre_msg
- * with the given sub-buffer lengths. */
+ * with the given sub-buffer lengths.
+ * NOTE: this should only be used for NEW requests, and should always be
+ *       in the form of a v2 request.  If this is a connection to a v1
+ *       target then the first buffer will be stripped because the ptlrpc
+ *       data is part of the lustre_msg_v1 header. b=14043 */
 int lustre_msg_size(__u32 magic, int count, int *lens)
 {
         int size[] = { sizeof(struct ptlrpc_body) };
@@ -133,6 +146,24 @@ int lustre_msg_size(__u32 magic, int count, int *lens)
         }
 }
 
+/* This is used to determine the size of a buffer that was already packed
+ * and will correctly handle the different message formats. */
+int lustre_packed_msg_size(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1: {
+                struct lustre_msg_v1 *v1_msg = (struct lustre_msg_v1 *)msg;
+                return lustre_msg_size_v1(v1_msg->lm_bufcount,
+                                          v1_msg->lm_buflens);
+        }
+        case LUSTRE_MSG_MAGIC_V2:
+                return lustre_msg_size_v2(msg->lm_bufcount, msg->lm_buflens);
+        default:
+                CERROR("incorrect message magic: %08x\n", msg->lm_magic);
+                return 0;
+        }
+}
+
 static void
 lustre_init_msg_v1(void *m, int count, int *lens, char **bufs)
 {
@@ -330,44 +361,7 @@ out:
 }
 
 static int lustre_pack_reply_v1(struct ptlrpc_request *req, int count,
-                                int *lens, char **bufs)
-{
-        struct ptlrpc_reply_state *rs;
-        int                        msg_len;
-        int                        size;
-        ENTRY;
-
-        LASSERT (req->rq_reply_state == NULL);
-
-        msg_len = lustre_msg_size_v1(count, lens);
-        size = sizeof(struct ptlrpc_reply_state) + msg_len;
-        OBD_ALLOC(rs, size);
-        if (unlikely(rs == NULL)) {
-                rs = lustre_get_emerg_rs(req->rq_rqbd->rqbd_service, size);
-                if (!rs)
-                        RETURN (-ENOMEM);
-        }
-        atomic_set(&rs->rs_refcount, 1);        /* 1 ref for rq_reply_state */
-        rs->rs_cb_id.cbid_fn = reply_out_callback;
-        rs->rs_cb_id.cbid_arg = rs;
-        rs->rs_service = req->rq_rqbd->rqbd_service;
-        rs->rs_size = size;
-        CFS_INIT_LIST_HEAD(&rs->rs_exp_list);
-        CFS_INIT_LIST_HEAD(&rs->rs_obd_list);
-        rs->rs_msg = (struct lustre_msg *)(rs + 1);
-
-        req->rq_replen = msg_len;
-        req->rq_reply_state = rs;
-        req->rq_repmsg = rs->rs_msg;
-        lustre_init_msg_v1(rs->rs_msg, count, lens, bufs);
-
-        PTLRPC_RS_DEBUG_LRU_ADD(rs);
-
-        RETURN (0);
-}
-
-static int lustre_pack_reply_v2(struct ptlrpc_request *req, int count,
-                                int *lens, char **bufs)
+                                int *lens, char **bufs, int flags)
 {
         struct ptlrpc_reply_state *rs;
         int                        msg_len;
@@ -376,12 +370,15 @@ static int lustre_pack_reply_v2(struct ptlrpc_request *req, int count,
 
         LASSERT(req->rq_reply_state == NULL);
 
-        msg_len = lustre_msg_size_v2(count, lens);
+        if ((flags & LPRFL_EARLY_REPLY) == 0)
+                req->rq_packed_final = 1;
+
+        msg_len = lustre_msg_size_v1(count, lens);
         size = sizeof(struct ptlrpc_reply_state) + msg_len;
         OBD_ALLOC(rs, size);
         if (unlikely(rs == NULL)) {
                 rs = lustre_get_emerg_rs(req->rq_rqbd->rqbd_service, size);
-                if (!rs)
+                if (!rs) 
                         RETURN (-ENOMEM);
         }
         atomic_set(&rs->rs_refcount, 1);        /* 1 ref for rq_reply_state */
@@ -396,6 +393,49 @@ static int lustre_pack_reply_v2(struct ptlrpc_request *req, int count,
         req->rq_replen = msg_len;
         req->rq_reply_state = rs;
         req->rq_repmsg = rs->rs_msg;
+
+        lustre_init_msg_v1(rs->rs_msg, count, lens, bufs);
+
+        PTLRPC_RS_DEBUG_LRU_ADD(rs);
+
+        RETURN (0);
+}
+
+static int lustre_pack_reply_v2(struct ptlrpc_request *req, int count,
+                                int *lens, char **bufs, int flags)
+{
+        struct ptlrpc_reply_state *rs;
+        int                        msg_len;
+        int                        size;
+        ENTRY;
+
+        LASSERT(req->rq_reply_state == NULL);
+
+        if ((flags & LPRFL_EARLY_REPLY) == 0)
+                req->rq_packed_final = 1;
+
+        msg_len = lustre_msg_size_v2(count, lens);
+        size = sizeof(struct ptlrpc_reply_state) + msg_len;
+        OBD_ALLOC(rs, size);
+        if (unlikely(rs == NULL)) {
+                rs = lustre_get_emerg_rs(req->rq_rqbd->rqbd_service, size);
+                if (!rs) 
+                        RETURN (-ENOMEM);
+        }
+        atomic_set(&rs->rs_refcount, 1);        /* 1 ref for rq_reply_state */
+        rs->rs_cb_id.cbid_fn = reply_out_callback;
+        rs->rs_cb_id.cbid_arg = rs;
+        rs->rs_service = req->rq_rqbd->rqbd_service;
+        rs->rs_size = size;
+        CFS_INIT_LIST_HEAD(&rs->rs_exp_list);
+        CFS_INIT_LIST_HEAD(&rs->rs_obd_list);
+        rs->rs_msg = (struct lustre_msg *)(rs + 1);
+
+        req->rq_replen = msg_len;
+        req->rq_reply_state = rs;
+        req->rq_repmsg = rs->rs_msg;
+
+        /* server side, no rq_repbuf */
         lustre_init_msg_v2(rs->rs_msg, count, lens, bufs);
         lustre_msg_add_version(rs->rs_msg, PTLRPC_MSG_VERSION);
         lustre_set_rep_swabbed(req, MSG_PTLRPC_BODY_OFF);
@@ -405,8 +445,8 @@ static int lustre_pack_reply_v2(struct ptlrpc_request *req, int count,
         RETURN(0);
 }
 
-int lustre_pack_reply(struct ptlrpc_request *req, int count, int *lens,
-                      char **bufs)
+int lustre_pack_reply_flags(struct ptlrpc_request *req, int count, int *lens,
+                            char **bufs, int flags)
 {
         int size[] = { sizeof(struct ptlrpc_body) };
 
@@ -422,10 +462,10 @@ int lustre_pack_reply(struct ptlrpc_request *req, int count, int *lens,
         case LUSTRE_MSG_MAGIC_V1:
         case LUSTRE_MSG_MAGIC_V1_SWABBED:
                 return lustre_pack_reply_v1(req, count - 1, lens + 1,
-                                            bufs ? bufs + 1 : NULL);
+                                            bufs ? bufs + 1 : NULL, flags);
         case LUSTRE_MSG_MAGIC_V2:
         case LUSTRE_MSG_MAGIC_V2_SWABBED:
-                return lustre_pack_reply_v2(req, count, lens, bufs);
+                return lustre_pack_reply_v2(req, count, lens, bufs, flags);
         default:
                 LASSERTF(0, "incorrect message magic: %08x\n",
                          req->rq_reqmsg->lm_magic);
@@ -433,6 +473,17 @@ int lustre_pack_reply(struct ptlrpc_request *req, int count, int *lens,
         }
 }
 
+int lustre_pack_reply(struct ptlrpc_request *req, int count, int *lens,
+                      char **bufs)
+{
+        int rc = lustre_pack_reply_flags(req, count, lens, bufs, 0);
+        if (rc != 0)
+                CERROR("lustre_pack_reply failed: rc=%d size=%d\n", rc,
+                       lustre_msg_size(req->rq_reqmsg->lm_magic, count, lens));
+        return rc;
+}
+
+
 void *lustre_msg_buf_v1(void *msg, int n, int min_size)
 {
         struct lustre_msg_v1 *m = (struct lustre_msg_v1 *)msg;
@@ -543,7 +594,7 @@ void lustre_shrink_reply_v1(struct ptlrpc_request *req, int segment,
                 msg->lm_buflens[msg->lm_bufcount - 1] = 0;
         }
 
-        req->rq_replen = lustre_msg_size_v1(msg->lm_bufcount, msg->lm_buflens);
+        req->rq_replen = lustre_msg_size_v1(msg->lm_bufcount, (int *)msg->lm_buflens);
 }
 
 void lustre_shrink_reply_v2(struct ptlrpc_request *req, int segment,
@@ -582,7 +633,7 @@ void lustre_shrink_reply_v2(struct ptlrpc_request *req, int segment,
                 msg->lm_buflens[msg->lm_bufcount - 1] = 0;
         }
 
-        req->rq_replen = lustre_msg_size_v2(msg->lm_bufcount, msg->lm_buflens);
+        req->rq_replen = lustre_msg_size_v2(msg->lm_bufcount, (int *)msg->lm_buflens);
 }
 
 /*
@@ -719,7 +770,7 @@ static int lustre_unpack_msg_v2(struct lustre_msg_v2 *m, int len)
                 __swab32s(&m->lm_secflvr);
                 __swab32s(&m->lm_repsize);
                 __swab32s(&m->lm_cksum);
-                CLASSERT(offsetof(typeof(*m), lm_padding_1) != 0);
+                __swab32s(&m->lm_flags);
                 CLASSERT(offsetof(typeof(*m), lm_padding_2) != 0);
                 CLASSERT(offsetof(typeof(*m), lm_padding_3) != 0);
         }
@@ -791,16 +842,18 @@ static inline int lustre_unpack_ptlrpc_body_v2(struct lustre_msg_v2 *m,
 {
         struct ptlrpc_body *pb;
 
-        pb = lustre_swab_buf(m, offset, sizeof(*pb), lustre_swab_ptlrpc_body);
+        pb = lustre_msg_buf_v2(m, offset, sizeof(*pb));
         if (!pb) {
-                CERROR("error unpacking ptlrpc body");
+                CERROR("error unpacking ptlrpc body\n");
                 return -EFAULT;
         }
+        if (lustre_msg_swabbed(m))
+                lustre_swab_ptlrpc_body(pb);
 
         if ((pb->pb_version & ~LUSTRE_VERSION_MASK) != PTLRPC_MSG_VERSION) {
                  CERROR("wrong lustre_msg version %08x\n", pb->pb_version);
                  return -EINVAL;
-         }
+        }
 
         return 0;
 }
@@ -1016,17 +1069,46 @@ void *lustre_swab_buf(struct lustre_msg *msg, int index, int min_size,
 void *lustre_swab_reqbuf(struct ptlrpc_request *req, int index, int min_size,
                          void *swabber)
 {
-        LASSERT_REQSWAB(req, index);
+        lustre_set_req_swabbed(req, index);
         return lustre_swab_buf(req->rq_reqmsg, index, min_size, swabber);
 }
 
 void *lustre_swab_repbuf(struct ptlrpc_request *req, int index, int min_size,
                          void *swabber)
 {
-        LASSERT_REPSWAB(req, index);
+        lustre_set_rep_swabbed(req, index);
         return lustre_swab_buf(req->rq_repmsg, index, min_size, swabber);
 }
 
+__u32 lustre_msghdr_get_flags(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+        case LUSTRE_MSG_MAGIC_V1_SWABBED:
+                return 0;
+        case LUSTRE_MSG_MAGIC_V2:
+        case LUSTRE_MSG_MAGIC_V2_SWABBED:
+                /* already in host endian */
+                return msg->lm_flags;
+        default:
+                LASSERTF(0, "incorrect message magic: %08x\n", msg->lm_magic);
+                return 0;
+        }
+}
+
+void lustre_msghdr_set_flags(struct lustre_msg *msg, __u32 flags)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+                return;
+        case LUSTRE_MSG_MAGIC_V2:
+                msg->lm_flags = flags;
+                return;
+        default:
+                LASSERTF(0, "incorrect message magic: %08x\n", msg->lm_magic);
+        }
+}
+
 __u32 lustre_msg_get_flags(struct lustre_msg *msg)
 {
         switch (msg->lm_magic) {
@@ -1503,6 +1585,17 @@ __u32 lustre_msg_get_conn_cnt(struct lustre_msg *msg)
         }
 }
 
+int lustre_msg_is_v1(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+        case LUSTRE_MSG_MAGIC_V1_SWABBED:
+                return 1;
+        default:
+                return 0;
+        }
+}
+
 __u32 lustre_msg_get_magic(struct lustre_msg *msg)
 {
         switch (msg->lm_magic) {
@@ -1517,6 +1610,88 @@ __u32 lustre_msg_get_magic(struct lustre_msg *msg)
         }
 }
 
+__u32 lustre_msg_get_timeout(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+        case LUSTRE_MSG_MAGIC_V1_SWABBED:
+                return 0;
+        case LUSTRE_MSG_MAGIC_V2:
+        case LUSTRE_MSG_MAGIC_V2_SWABBED: {
+                struct ptlrpc_body *pb;
+                
+                pb = lustre_msg_buf_v2(msg, MSG_PTLRPC_BODY_OFF, sizeof(*pb));
+                if (!pb) {
+                        CERROR("invalid msg %p: no ptlrpc body!\n", msg);
+                        return 0;
+                        
+                }
+                return pb->pb_timeout;
+        }
+        default:
+                CERROR("incorrect message magic: %08x\n", msg->lm_magic);
+                return 0;
+        }
+}
+
+__u32 lustre_msg_get_service_time(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+        case LUSTRE_MSG_MAGIC_V1_SWABBED:
+                return 0;
+        case LUSTRE_MSG_MAGIC_V2:
+        case LUSTRE_MSG_MAGIC_V2_SWABBED: {
+                struct ptlrpc_body *pb;
+        
+                pb = lustre_msg_buf_v2(msg, MSG_PTLRPC_BODY_OFF, sizeof(*pb));
+                if (!pb) {
+                        CERROR("invalid msg %p: no ptlrpc body!\n", msg);
+                        return 0;
+                        
+                }
+                return pb->pb_service_time;
+        }
+        default:
+                CERROR("incorrect message magic: %08x\n", msg->lm_magic);
+                return 0;
+        }
+}
+
+__u32 lustre_msg_get_cksum(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+        case LUSTRE_MSG_MAGIC_V1_SWABBED:
+                return 0;
+        case LUSTRE_MSG_MAGIC_V2:
+        case LUSTRE_MSG_MAGIC_V2_SWABBED:
+                return msg->lm_cksum;
+        default:
+                CERROR("incorrect message magic: %08x\n", msg->lm_magic);
+                return 0;
+        }
+}
+
+__u32 lustre_msg_calc_cksum(struct lustre_msg *msg)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+        case LUSTRE_MSG_MAGIC_V1_SWABBED:
+                return 0;
+        case LUSTRE_MSG_MAGIC_V2:
+        case LUSTRE_MSG_MAGIC_V2_SWABBED: {
+                struct ptlrpc_body *pb;
+                pb = lustre_msg_buf_v2(msg, MSG_PTLRPC_BODY_OFF, sizeof(*pb));
+                LASSERTF(pb, "invalid msg %p: no ptlrpc body!\n", msg);
+                return crc32_le(~(__u32)0, (char *)pb, sizeof(*pb));
+        }
+        default:
+                CERROR("incorrect message magic: %08x\n", msg->lm_magic);
+                return 0;
+        }
+}
+
 void lustre_msg_set_handle(struct lustre_msg *msg, struct lustre_handle *handle)
 {
         switch (msg->lm_magic) {
@@ -1669,6 +1844,56 @@ void lustre_msg_set_conn_cnt(struct lustre_msg *msg, __u32 conn_cnt)
         }
 }
 
+void lustre_msg_set_timeout(struct lustre_msg *msg, __u32 timeout)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+                return;
+        case LUSTRE_MSG_MAGIC_V2: {
+                struct ptlrpc_body *pb;
+                
+                pb = lustre_msg_buf_v2(msg, MSG_PTLRPC_BODY_OFF, sizeof(*pb));
+                LASSERTF(pb, "invalid msg %p: no ptlrpc body!\n", msg);
+                pb->pb_timeout = timeout;
+                return;
+        }
+        default:
+                LASSERTF(0, "incorrect message magic: %08x\n", msg->lm_magic);
+        }
+}
+
+void lustre_msg_set_service_time(struct lustre_msg *msg, __u32 service_time)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+                return;
+        case LUSTRE_MSG_MAGIC_V2: {
+                struct ptlrpc_body *pb;
+
+                pb = lustre_msg_buf_v2(msg, MSG_PTLRPC_BODY_OFF, sizeof(*pb));
+                LASSERTF(pb, "invalid msg %p: no ptlrpc body!\n", msg);
+                pb->pb_service_time = service_time;
+                return;
+        }
+        default:
+                LASSERTF(0, "incorrect message magic: %08x\n", msg->lm_magic);
+        }
+}
+
+void lustre_msg_set_cksum(struct lustre_msg *msg, __u32 cksum)
+{
+        switch (msg->lm_magic) {
+        case LUSTRE_MSG_MAGIC_V1:
+                return;
+        case LUSTRE_MSG_MAGIC_V2:
+                msg->lm_cksum = cksum;
+                return;
+        default:
+                LASSERTF(0, "incorrect message magic: %08x\n", msg->lm_magic);
+        }
+}
+
+
 /* byte flipping routines for all wire types declared in
  * lustre_idl.h implemented here.
  */
@@ -1701,10 +1926,11 @@ void lustre_swab_connect(struct obd_connect_data *ocd)
         __swab64s(&ocd->ocd_ibits_known);
         __swab32s(&ocd->ocd_nllu);
         __swab32s(&ocd->ocd_nllg);
+        __swab64s(&ocd->ocd_transno);
+        __swab32s(&ocd->ocd_group);
+        __swab32s(&ocd->ocd_cksum_types);
         CLASSERT(offsetof(typeof(*ocd), padding1) != 0);
         CLASSERT(offsetof(typeof(*ocd), padding2) != 0);
-        CLASSERT(offsetof(typeof(*ocd), padding3) != 0);
-        CLASSERT(offsetof(typeof(*ocd), padding4) != 0);
 }
 
 void lustre_swab_obdo (struct obdo  *o)
@@ -1866,6 +2092,14 @@ void lustre_swab_obd_quotactl (struct obd_quotactl *q)
         lustre_swab_obd_dqblk (&q->qc_dqblk);
 }
 
+void lustre_swab_quota_adjust_qunit (struct quota_adjust_qunit *q)
+{
+        __swab32s (&q->qaq_flags);
+        __swab32s (&q->qaq_id);
+        __swab64s (&q->qaq_bunit_sz);
+        __swab64s (&q->qaq_iunit_sz);
+}
+
 void lustre_swab_mds_rec_setattr (struct mds_rec_setattr *sa)
 {
         __swab32s (&sa->sa_opcode);
@@ -2135,8 +2369,10 @@ void lustre_swab_qdata(struct qunit_data *d)
         __swab32s (&d->qd_id);
         __swab32s (&d->qd_flags);
         __swab64s (&d->qd_count);
+        __swab64s (&d->qd_qunit);
 }
 
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 7, 0, 0)
 void lustre_swab_qdata_old(struct qunit_data_old *d)
 {
         __swab32s (&d->qd_id);
@@ -2144,28 +2380,39 @@ void lustre_swab_qdata_old(struct qunit_data_old *d)
         __swab32s (&d->qd_count);
         __swab32s (&d->qd_isblk);
 }
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
+void lustre_swab_qdata_old2(struct qunit_data_old2 *d)
+{
+        __swab32s (&d->qd_id);
+        __swab32s (&d->qd_flags);
+        __swab64s (&d->qd_count);
+}
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
 
 #ifdef __KERNEL__
-struct qunit_data *lustre_quota_old_to_new(struct qunit_data_old *d)
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 7, 0, 0)
+void qdata_v1_v3(struct qunit_data_old *d,
+                             struct qunit_data *qdata)
 {
-        struct qunit_data_old tmp;
-        struct qunit_data *ret;
-        ENTRY;
-
-        if (!d)
-                return NULL;
-
-        tmp = *d;
-        ret = (struct qunit_data *)d;
-        ret->qd_id = tmp.qd_id;
-        ret->qd_flags = (tmp.qd_type ? QUOTA_IS_GRP : 0) | (tmp.qd_isblk ? QUOTA_IS_BLOCK : 0);
-        ret->qd_count = tmp.qd_count;
-        RETURN(ret);
+        LASSERT(d);
+        LASSERT(qdata);
 
+        qdata->qd_id = d->qd_id;
+        if (d->qd_type)
+                QDATA_SET_GRP(qdata);
+        if (d->qd_isblk)
+                QDATA_SET_BLK(qdata);
+        qdata->qd_count = d->qd_count;
 }
-EXPORT_SYMBOL(lustre_quota_old_to_new);
 
-struct qunit_data_old *lustre_quota_new_to_old(struct qunit_data *d)
+struct qunit_data_old *qdata_v3_to_v1(struct qunit_data *d)
 {
         struct qunit_data tmp;
         struct qunit_data_old *ret;
@@ -2177,12 +2424,239 @@ struct qunit_data_old *lustre_quota_new_to_old(struct qunit_data *d)
         tmp = *d;
         ret = (struct qunit_data_old *)d;
         ret->qd_id = tmp.qd_id;
-        ret->qd_type = ((tmp.qd_flags & QUOTA_IS_GRP) ? GRPQUOTA : USRQUOTA);
+        ret->qd_type = (QDATA_IS_GRP(&tmp) ? GRPQUOTA : USRQUOTA);
         ret->qd_count = (__u32)tmp.qd_count;
-        ret->qd_isblk = ((tmp.qd_flags & QUOTA_IS_BLOCK) ? 1 : 0);
+        ret->qd_isblk = (QDATA_IS_BLK(&tmp) ? 1 : 0);
         RETURN(ret);
 }
-EXPORT_SYMBOL(lustre_quota_new_to_old);
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
+void qdata_v2_to_v3(struct qunit_data_old2 *d,
+                              struct qunit_data *qdata)
+{
+        LASSERT(d);
+        LASSERT(qdata);
+
+        qdata->qd_id = d->qd_id;
+        qdata->qd_flags = d->qd_flags;
+        qdata->qd_count = d->qd_count;
+}
+
+struct qunit_data_old2 *qdata_v3_to_v2(struct qunit_data *d)
+{
+        struct qunit_data tmp;
+        struct qunit_data_old2 *ret;
+        ENTRY;
+
+        if (!d)
+                return NULL;
+
+        tmp = *d;
+        ret = (struct qunit_data_old2 *)d;
+        ret->qd_id = tmp.qd_id;
+        ret->qd_flags = tmp.qd_flags & LQUOTA_QUNIT_FLAGS;
+        ret->qd_count = tmp.qd_count;
+        RETURN(ret);
+}
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+/* got qdata from request(req/rep) */
+int quota_get_qdata(void *request, struct qunit_data *qdata,
+                    int is_req, int is_exp)
+{
+        struct ptlrpc_request *req = (struct ptlrpc_request *)request;
+        struct qunit_data *new;
+        struct qunit_data_old *old;
+        struct qunit_data_old2 *old2;
+        int size  = sizeof(struct qunit_data_old);
+        int size2 = sizeof(struct qunit_data_old2);
+        __u64  flags = is_exp ? req->rq_export->exp_connect_flags :
+                       req->rq_import->imp_connect_data.ocd_connect_flags;
+
+        LASSERT(req);
+        LASSERT(qdata);
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 7, 0, 0)
+        if (OBD_FAIL_CHECK(OBD_FAIL_QUOTA_QD_COUNT_32BIT))
+                goto quota32;
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
+        if (OBD_FAIL_CHECK(OBD_FAIL_QUOTA_WITHOUT_CHANGE_QS))
+                goto without_change_qs;
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+        /* support for quota64 and change_qs */
+        if (flags & OBD_CONNECT_CHANGE_QS) {
+                if (!(flags & OBD_CONNECT_QUOTA64)) {
+                        CDEBUG(D_ERROR, "Wire protocol for qunit is broken!\n");
+                        return -EINVAL;
+                }
+                if (is_req == QUOTA_REQUEST)
+                        new = lustre_swab_reqbuf(req, REQ_REC_OFF,
+                                                 sizeof(struct qunit_data),
+                                                 lustre_swab_qdata);
+                else
+                        new = lustre_swab_repbuf(req, REPLY_REC_OFF,
+                                                 sizeof(struct qunit_data),
+                                                 lustre_swab_qdata);
+                *qdata = *new;
+                QDATA_SET_CHANGE_QS(qdata);
+                return 0;
+        } else {
+                QDATA_CLR_CHANGE_QS(qdata);
+        }
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
+without_change_qs:
+        /* only support for quota64 */
+        if (flags & OBD_CONNECT_QUOTA64) {
+
+                if (is_req == QUOTA_REQUEST)
+                        old2 = lustre_swab_reqbuf(req, REQ_REC_OFF, size2,
+                                                  lustre_swab_qdata_old2);
+                else
+                        old2 = lustre_swab_repbuf(req, REPLY_REC_OFF, size2,
+                                                  lustre_swab_qdata_old2);
+                qdata_v2_to_v3(old2, qdata);
+
+                return 0;
+        }
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 7, 0, 0)
+quota32:
+        /* not support for quota64 and change_qs */
+        if (is_req == QUOTA_REQUEST)
+                old = lustre_swab_reqbuf(req, REQ_REC_OFF, size,
+                                         lustre_swab_qdata_old);
+        else
+                old = lustre_swab_repbuf(req, REPLY_REC_OFF, size,
+                                         lustre_swab_qdata_old);
+        qdata_v1_v3(old, qdata);
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+        return 0;
+}
+EXPORT_SYMBOL(quota_get_qdata);
+
+/* copy qdata to request(req/rep) */
+int quota_copy_qdata(void *request, struct qunit_data *qdata,
+                     int is_req, int is_exp)
+{
+        struct ptlrpc_request *req = (struct ptlrpc_request *)request;
+        void *target;
+        struct qunit_data_old *old;
+        struct qunit_data_old2 *old2;
+        __u64  flags = is_exp ? req->rq_export->exp_connect_flags :
+                req->rq_import->imp_connect_data.ocd_connect_flags;
+
+        LASSERT(req);
+        LASSERT(qdata);
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 7, 0, 0)
+        if (OBD_FAIL_CHECK(OBD_FAIL_QUOTA_QD_COUNT_32BIT))
+                goto quota32;
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
+        if (OBD_FAIL_CHECK(OBD_FAIL_QUOTA_WITHOUT_CHANGE_QS))
+                goto without_change_qs;
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+        /* support for quota64 and change_qs */
+        if (flags & OBD_CONNECT_CHANGE_QS) {
+                if (!(flags & OBD_CONNECT_QUOTA64)) {
+                        CERROR("Wire protocol for qunit is broken!\n");
+                        return -EINVAL;
+                }
+                if (is_req == QUOTA_REQUEST)
+                        target = lustre_msg_buf(req->rq_reqmsg, REQ_REC_OFF,
+                                                sizeof(struct qunit_data));
+                else
+                        target = lustre_msg_buf(req->rq_repmsg, REPLY_REC_OFF,
+                                                sizeof(struct qunit_data));
+                if (!target)
+                        return -EINVAL;
+                memcpy(target, qdata, sizeof(*qdata));
+                return 0;
+        }
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
+without_change_qs:
+        /* only support for quota64 */
+        if (flags & OBD_CONNECT_QUOTA64) {
+                if (is_req == QUOTA_REQUEST)
+                        target = lustre_msg_buf(req->rq_reqmsg, REQ_REC_OFF,
+                                                sizeof(struct qunit_data_old2));
+                else
+                        target = lustre_msg_buf(req->rq_repmsg, REPLY_REC_OFF,
+                                                sizeof(struct qunit_data_old2));
+                if (!target)
+                        return -EINVAL;
+                old2 = qdata_v3_to_v2(qdata);
+                memcpy(target, old2, sizeof(*old2));
+                return 0;
+        }
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 7, 0, 0)
+quota32:
+        /* not support for quota64 and change_qs */
+        if (is_req == QUOTA_REQUEST)
+                target = lustre_msg_buf(req->rq_reqmsg, REQ_REC_OFF,
+                                        sizeof(struct qunit_data_old));
+        else
+                target = lustre_msg_buf(req->rq_repmsg, REPLY_REC_OFF,
+                                        sizeof(struct qunit_data_old));
+        if (!target)
+                return -EINVAL;
+        old = qdata_v3_to_v1(qdata);
+        memcpy(target, old, sizeof(*old));
+#else
+#warning "remove quota code above for format absolete in new release"
+#endif
+
+        return 0;
+}
+EXPORT_SYMBOL(quota_copy_qdata);
+
+int quota_get_qunit_data_size(__u64 flag)
+{
+        int size;
+
+        if (flag & OBD_CONNECT_CHANGE_QS) {
+                size = sizeof(struct qunit_data);
+        } else {
+                /* write in this way because sizes of qunit_data_old and
+                 * qunit_data_old2 are same */
+                LASSERT(sizeof(struct qunit_data_old) ==
+                        sizeof(struct qunit_data_old2));
+                size = sizeof(struct qunit_data_old);
+        }
+
+        return(size);
+}
+EXPORT_SYMBOL(quota_get_qunit_data_size);
 #endif /* __KERNEL__ */
 
 static inline int req_ptlrpc_body_swabbed(struct ptlrpc_request *req)
@@ -2229,8 +2703,9 @@ void _debug_req(struct ptlrpc_request *req, __u32 mask,
         va_start(args, fmt);
         libcfs_debug_vmsg2(data->msg_cdls, data->msg_subsys, mask, data->msg_file,
                            data->msg_fn, data->msg_line, fmt, args,
-                           " req@%p x"LPD64"/t"LPD64" o%d->%s@%s:%d lens %d/%d ref %d fl "
-                           REQ_FLAGS_FMT"/%x/%x rc %d/%d\n",
+                           " req@%p x"LPD64"/t"LPD64" o%d->%s@%s:%d/%d "
+                           "lens %d/%d e %d to %d dl %ld ref %d "
+                           "fl "REQ_FLAGS_FMT"/%x/%x rc %d/%d\n",
                            req, req->rq_xid, req->rq_transno,
                            req->rq_reqmsg ? lustre_msg_get_opc(req->rq_reqmsg) : -1,
                            req->rq_import ? obd2cli_tgt(req->rq_import->imp_obd) :
@@ -2240,10 +2715,10 @@ void _debug_req(struct ptlrpc_request *req, __u32 mask,
                                 (char *)req->rq_import->imp_connection->c_remote_uuid.uuid :
                            req->rq_export ?
                                 (char *)req->rq_export->exp_connection->c_remote_uuid.uuid : "<?>",
-                           (req->rq_import && req->rq_import->imp_client) ?
-                                req->rq_import->imp_client->cli_request_portal : -1,
-                           req->rq_reqlen, req->rq_replen, atomic_read(&req->rq_refcount),
-                           DEBUG_REQ_FLAGS(req),
+                           req->rq_request_portal,  req->rq_reply_portal,
+                           req->rq_reqlen, req->rq_replen,
+                           req->rq_early_count, req->rq_timeout, req->rq_deadline,
+                           atomic_read(&req->rq_refcount), DEBUG_REQ_FLAGS(req),
                            req->rq_reqmsg ? lustre_msg_get_flags(req->rq_reqmsg) : 0,
                            req->rq_repmsg ? lustre_msg_get_flags(req->rq_repmsg) : 0,
                            req->rq_status,
