@@ -6,6 +6,7 @@ mdsfailover_HOST=${mdsfailover_HOST}
 mgs_HOST=${mgs_HOST:-$mds_HOST}
 ost_HOST=${ost_HOST:-`hostname`}
 ostfailover_HOST=${ostfailover_HOST}
+PDSH=${PDSH:-no_dsh}
 
 TMP=${TMP:-/tmp}
 
@@ -32,6 +33,8 @@ PTLDEBUG=${PTLDEBUG:-0x33f1504}
 DEBUG_SIZE=${DEBUG_SIZE:-10}
 SUBSYSTEM=${SUBSYSTEM:- 0xffb7e3ff}
 
+L_GETGROUPS=${L_GETGROUPS:-`do_facet mds which l_getgroups || echo`}
+
 MKFSOPT=""
 MOUNTOPT=""
 [ "x$MDSJOURNALSIZE" != "x" ] &&
@@ -46,8 +49,8 @@ MOUNTOPT=""
     MOUNTOPT=$MOUNTOPT" --param lov.stripesize=$STRIPE_BYTES"
 [ "x$STRIPES_PER_OBJ" != "x" ] &&
     MOUNTOPT=$MOUNTOPT" --param lov.stripecount=$STRIPES_PER_OBJ"
-[ "x$LUSTRE" != "x" ] && [ -f $LUSTRE/utils/l_getgroups ] &&
-    MOUNTOPT=$MOUNTOPT" --param mdt.group_upcall=$LUSTRE/utils/l_getgroups"
+[ "x$L_GETGROUPS" != "x" ] &&
+    MOUNTOPT=$MOUNTOPT" --param mdt.group_upcall=$L_GETGROUPS"
 MDS_MKFS_OPTS="--mgs --mdt --fsname=$FSNAME --device-size=$MDSSIZE --param sys.timeout=$TIMEOUT $MKFSOPT $MOUNTOPT $MDSOPT"
 
 MKFSOPT=""
@@ -55,7 +58,7 @@ MOUNTOPT=""
 [ "x$OSTJOURNALSIZE" != "x" ] &&
     MKFSOPT=$MKFSOPT" -J size=$OSTJOURNALSIZE"
 [ "x$MKFSOPT" != "x" ] &&
-    MKFSOPT="--mkfsoptions=\"$MKFSOPT\""
+    MKFSOPT="--mkfsoptions=\\\"$MKFSOPT\\\""
 [ "x$ostfailover_HOST" != "x" ] &&
     MOUNTOPT=$MOUNTOPT" --failnode=`h2$NETTYPE $ostfailover_HOST`"
 OST_MKFS_OPTS="--ost --fsname=$FSNAME --device-size=$OSTSIZE --mgsnode=$MGSNID --param sys.timeout=$TIMEOUT $MKFSOPT $MOUNTOPT $OSTOPT"
@@ -72,7 +75,17 @@ DIR=${DIR:-$MOUNT}
 DIR1=${DIR:-$MOUNT1}
 DIR2=${DIR2:-$MOUNT2}
 
-PDSH=${PDSH:-no_dsh}
+if [ $UID -ne 0 ]; then
+        log "running as non-root uid $UID"
+        RUNAS_ID="$UID"
+        RUNAS=""
+else
+        RUNAS_ID=${RUNAS_ID:-500}
+        RUNAS=${RUNAS:-"runas -u $RUNAS_ID"}
+fi
+
 FAILURE_MODE=${FAILURE_MODE:-SOFT} # or HARD
 POWER_DOWN=${POWER_DOWN:-"powerman --off"}
 POWER_UP=${POWER_UP:-"powerman --on"}
+SLOW=${SLOW:-no}
+FAIL_ON_ERROR=${FAIL_ON_ERROR:-true}
