@@ -15,8 +15,8 @@ ONLY=${ONLY:-"$*"}
 #              xml xml xml xml xml xml dumb
 MOUNTCONFSKIP="10  11  12  13  13b 14  15 "
 
-# bug number for skipped test:                     12743
-ALWAYS_EXCEPT=" $CONF_SANITY_EXCEPT $MOUNTCONFSKIP 36"
+# bug number for skipped test:                     13369
+ALWAYS_EXCEPT=" $CONF_SANITY_EXCEPT $MOUNTCONFSKIP 34a"
 # UPDATE THE COMMENT ABOVE WITH BUG NUMBERS WHEN CHANGING ALWAYS_EXCEPT!
 
 SRCDIR=`dirname $0`
@@ -34,6 +34,9 @@ MDSSIZE=40000
 OSTSIZE=40000
 . ${CONFIG:=$LUSTRE/tests/cfg/$NAME.sh}
 
+#
+[ "$SLOW" = "no" ] && EXCEPT_SLOW="0 1 2 3 6 7 15 18 24b 25 30 31 32 33 34a "
+
 reformat() {
         formatall
 }
@@ -232,7 +235,7 @@ test_4() {
 }
 run_test 4 "force cleanup ost, then cleanup"
 
-test_5() {
+test_5a() {	# was test_5
 	setup
 	touch $DIR/$tfile || return 1
 	fuser -m -v $MOUNT && echo "$MOUNT is in use by user space process."
@@ -261,17 +264,25 @@ test_5() {
 		grep " $MOUNT " /proc/mounts && echo "test 5: /proc/mounts after second umount" && return 11
 	fi
 
-	sleep 0.500s
 	manual_umount_client
 	# stop_mds is a no-op here, and should not fail
 	cleanup_nocli || return $?
 	# df may have lingering entry
 	manual_umount_client
 	# mtab may have lingering entry
-	grep -v $MOUNT" " /etc/mtab > $TMP/mtabtemp
-	mv $TMP/mtabtemp /etc/mtab
+	local WAIT=0
+	local MAX_WAIT=20
+	local sleep=1
+	while [ "$WAIT" -ne "$MAX_WAIT" ]; do
+		sleep $sleep
+		grep -q $MOUNT" " /etc/mtab || break
+        	echo "Waiting /etc/mtab updated ... "
+		WAIT=$(( WAIT + sleep))
+	done
+	[ "$WAIT" -eq "$MAX_WAIT" ] && error "/etc/mtab is not updated in $WAIT secs"
+	echo "/etc/mtab updated in $WAIT secs"
 }
-run_test 5 "force cleanup mds, then cleanup"
+run_test 5a "force cleanup mds, then cleanup"
 
 test_5b() {
 	start_ost
@@ -291,7 +302,10 @@ test_5c() {
 	start_mds
 	[ -d $MOUNT ] || mkdir -p $MOUNT
 	grep " $MOUNT " /etc/mtab && echo "test 5c: mtab before mount" && return 10
-	mount -t lustre $MGSNID:/wrong.$FSNAME $MOUNT || :
+	local oldfs="${FSNAME}"
+	FSNAME="wrong.${FSNAME}"
+	mount_client $MOUNT || :
+	FSNAME=${oldfs}
 	grep " $MOUNT " /etc/mtab && echo "test 5c: mtab after failed mount" && return 11
 	umount_client $MOUNT
 	cleanup_nocli  || return $?
@@ -371,6 +385,7 @@ test_9() {
         fi
         stop_ost || return $?
 }
+
 run_test 9 "test ptldebug and subsystem for mkfs"
 
 test_10() {
@@ -627,7 +642,7 @@ test_15() {
 run_test 15 "zconf-mount without /sbin/mount.lustre (should return error)"
 
 test_16() {
-        TMPMTPT="${MOUNT%/*}/conf16"
+        TMPMTPT="${TMP}/conf16"
 
         if [ ! -e "$MDSDEV" ]; then
             log "no $MDSDEV existing, so mount Lustre to create one"
@@ -782,59 +797,110 @@ test_22() {
         #reformat to remove all logs
         reformat
 	start_mds
-
-	echo Client mount with a running ost
-	start_ost
+	echo Client mount before any osts are in the logs
 	mount_client $MOUNT
-	check_mount || return 41
-	umount_client $MOUNT
+	check_mount && return 41
 	pass
 
 	echo Client mount with ost in logs, but none running
+	start_ost
 	stop_ost
 	mount_client $MOUNT
 	# check_mount will block trying to contact ost
 	umount_client $MOUNT
 	pass
 
+	echo Client mount with a running ost
+	start_ost
+	mount_client $MOUNT
+	check_mount || return 41
+	pass
+
 	cleanup
 }
 run_test 22 "start a client before osts (should return errs)"
 
-test_23() {
+test_23a() {	# was test_23
         setup
         # fail mds
 	stop mds   
 	# force down client so that recovering mds waits for reconnect
-	zconf_umount `hostname` $MOUNT -f
+	local running=$(grep -c $MOUNT /proc/mounts) || true
+    	if [ $running -ne 0 ]; then
+        	echo "Stopping client $MOUNT (opts: -f)"
+        	umount -f $MOUNT
+    	fi
+
 	# enter recovery on mds
 	start_mds
 	# try to start a new client
 	mount_client $MOUNT &
-	MOUNT_PID=$!
 	sleep 5
+	MOUNT_PID=$(ps -ef | grep "t lustre" | grep -v grep | awk '{print $2}')
 	MOUNT_LUSTRE_PID=`ps -ef | grep mount.lustre | grep -v grep | awk '{print $2}'`
 	echo mount pid is ${MOUNT_PID}, mount.lustre pid is ${MOUNT_LUSTRE_PID}
 	ps --ppid $MOUNT_PID
 	ps --ppid $MOUNT_LUSTRE_PID
 	# FIXME why o why can't I kill these? Manual "ctrl-c" works...
-	kill -TERM $MOUNT_PID
+	kill -TERM $MOUNT_LUSTRE_PID
 	echo "waiting for mount to finish"
 	ps -ef | grep mount
-	wait $MOUNT_PID
+	# we can not wait $MOUNT_PID because it is not a child of this shell
+	local PID1
+	local PID2
+	local WAIT=0
+	local MAX_WAIT=20
+	local sleep=1
+	while [ "$WAIT" -lt "$MAX_WAIT" ]; do
+		sleep $sleep
+		PID1=$(ps -ef | awk '{print $2}' | grep -w $MOUNT_PID)
+		PID2=$(ps -ef | awk '{print $2}' | grep -w $MOUNT_LUSTRE_PID)
+		echo PID1=$PID1
+		echo PID2=$PID2
+		[ -z "$PID1" -a -z "$PID2" ] && break
+		echo "waiting for mount to finish ... "
+		WAIT=$(( WAIT + sleep))
+	done
+	[ "$WAIT" -eq "$MAX_WAIT" ] && error "MOUNT_PID $MOUNT_PID and \
+		MOUNT__LUSTRE_PID $MOUNT__LUSTRE_PID still not killed in $WAIT secs"
+	ps -ef | grep mount
+	stop_mds || error
+	stop_ost || error
+}
+run_test 23a "interrupt client during recovery mount delay"
 
-	stop_mds
-	stop_ost
+umount_client $MOUNT
+cleanup_nocli
+
+test_23b() {    # was test_23
+	start_ost
+	start_mds
+	# Simulate -EINTR during mount OBD_FAIL_LDLM_CLOSE_THREAD
+	sysctl -w lustre.fail_loc=0x80000313
+	mount_client $MOUNT
+	cleanup
+}
+run_test 23b "Simulate -EINTR during mount"
+
+fs2mds_HOST=$mds_HOST
+fs2ost_HOST=$ost_HOST
+
+cleanup_24a() {
+	trap 0
+	echo "umount $MOUNT2 ..."
+	umount $MOUNT2 || true
+	echo "stopping fs2mds ..."
+	stop fs2mds -f || true
+	echo "stopping fs2ost ..."
+	stop fs2ost -f || true
 }
-#this test isn't working yet
-#run_test 23 "interrupt client during recovery mount delay"
 
 test_24a() {
-	local fs2mds_HOST=$mds_HOST
-	local fs2ost_HOST=$ost_HOST
-
-	[ -z "$fs2ost_DEV" -o -z "$fs2mds_DEV" ] && [ -b "$MDSDEV" ] && \
-            log "mixed loopback and real device not working" && return
+	[ -n "$ost1_HOST" ] && fs2ost_HOST=$ost1_HOST
+	if [ -z "$fs2ost_DEV" -o -z "$fs2mds_DEV" ]; then
+		do_facet mds [ -b "$MDSDEV" ] && \
+		skip "mixed loopback and real device not working" && return
+	fi
 
 	local fs2mdsdev=${fs2mds_DEV:-${MDSDEV}_2}
 	local fs2ostdev=${fs2ost_DEV:-$(ostdevname 1)_2}
@@ -846,7 +912,7 @@ test_24a() {
 	add fs2ost $OST_MKFS_OPTS --fsname=${FSNAME2} --reformat $fs2ostdev || exit 10
 
 	setup
-	start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS
+	start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS && trap cleanup_24a EXIT INT
 	start fs2ost $fs2ostdev $OST_MOUNT_OPTS
 	mkdir -p $MOUNT2
 	mount -t lustre $MGSNID:/${FSNAME2} $MOUNT2 || return 1
@@ -868,23 +934,22 @@ test_24a() {
  	umount_client $MOUNT 
 	# the MDS must remain up until last MDT
 	stop_mds
-	MDS=$(do_facet mds "cat $LPROC/devices" | awk '($3 ~ "mdt" && $4 ~ "MDS") { print $4 }')
+	MDS=$(do_facet mds "lctl get_param -n devices" | awk '($3 ~ "mdt" && $4 ~ "MDS") { print $4 }')
 	[ -z "$MDS" ] && error "No MDS" && return 8
-	umount $MOUNT2
-	stop fs2mds -f
-	stop fs2ost -f
+	cleanup_24a
 	cleanup_nocli || return 6
 }
 run_test 24a "Multiple MDTs on a single node"
 
 test_24b() {
-	local fs2mds_HOST=$mds_HOST
-        [ -z "$fs2mds_DEV" ] && [ -b "$MDSDEV" ] && \
-            log "mixed loopback and real device not working" && return
+	if [ -z "$fs2mds_DEV" ]; then
+		do_facet mds [ -b "$MDSDEV" ] && \
+		skip "mixed loopback and real device not working" && return
+	fi
 
 	local fs2mdsdev=${fs2mds_DEV:-${MDSDEV}_2}
 
-        add fs2mds $MDS_MKFS_OPTS --fsname=${FSNAME}2 --mgs --reformat $fs2mdsdev || exit 10 
+	add fs2mds $MDS_MKFS_OPTS --fsname=${FSNAME}2 --mgs --reformat $fs2mdsdev || exit 10 
 	setup
 	start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS && return 2
 	cleanup || return 6
@@ -907,8 +972,8 @@ test_26() {
 #define OBD_FAIL_MDS_FS_SETUP            0x135
     do_facet mds "sysctl -w lustre.fail_loc=0x80000135"
     start_mds && echo MDS started && return 1
-    cat $LPROC/devices
-    DEVS=$(cat $LPROC/devices | wc -l)
+    lctl get_param -n devices
+    DEVS=$(lctl get_param -n devices | wc -l)
     [ $DEVS -gt 0 ] && return 2
     unload_modules || return 203
 }
@@ -951,7 +1016,7 @@ test_27a() {
 	start_mds || return 2
 	echo "Requeue thread should have started: " 
 	ps -e | grep ll_cfg_requeue 
-	set_and_check ost1 "cat $LPROC/obdfilter/$FSNAME-OST0000/client_cache_seconds" "$FSNAME-OST0000.ost.client_cache_seconds" || return 3 
+	set_and_check ost1 "lctl get_param -n obdfilter.$FSNAME-OST0000.client_cache_seconds" "$FSNAME-OST0000.ost.client_cache_seconds" || return 3
 	cleanup_nocli
 }
 run_test 27a "Reacquire MGS lock if OST started first"
@@ -959,17 +1024,15 @@ run_test 27a "Reacquire MGS lock if OST started first"
 test_27b() {
         setup
 	facet_failover mds
-	set_and_check mds "cat $LPROC/mds/$FSNAME-MDT0000/group_acquire_expire" "$FSNAME-MDT0000.mdt.group_acquire_expire" || return 3 
-	set_and_check client "cat $LPROC/mdc/$FSNAME-MDT0000-mdc-*/max_rpcs_in_flight" "$FSNAME-MDT0000.mdc.max_rpcs_in_flight" || return 4 
-	sleep 3
-	lctl mark "cleaning up"
+	set_and_check mds "lctl get_param -n mds.$FSNAME-MDT0000.group_acquire_expire" "$FSNAME-MDT0000.mdt.group_acquire_expire" || return 3
+	set_and_check client "lctl get_param -n mdc.$FSNAME-MDT0000-mdc-*.max_rpcs_in_flight" "$FSNAME-MDT0000.mdc.max_rpcs_in_flight" || return 4
 	cleanup
 }
 run_test 27b "Reacquire MGS lock after failover"
 
 test_28() {
         setup
-	TEST="cat $LPROC/llite/$FSNAME-*/max_read_ahead_whole_mb"
+	TEST="lctl get_param -n llite.$FSNAME-*.max_read_ahead_whole_mb"
 	ORIG=$($TEST) 
 	declare -i FINAL
 	FINAL=$(($ORIG + 10))
@@ -995,32 +1058,28 @@ test_29() {
 	sleep 10
 
 	local PARAM="$FSNAME-OST0001.osc.active"
-	local PROC_ACT="$LPROC/osc/$FSNAME-OST0001-osc-*/active"
-	local PROC_UUID="$LPROC/osc/$FSNAME-OST0001-osc-*/ost_server_uuid"
-	if [ ! -r $PROC_ACT ]; then
-	    echo "Can't read $PROC_ACT"
-	    ls $LPROC/osc/$FSNAME-*
-	    return 1
-	fi
-	ACTV=$(cat $PROC_ACT)
+	local PROC_ACT="osc.$FSNAME-OST0001-osc-*.active"
+	local PROC_UUID="osc.$FSNAME-OST0001-osc-*.ost_server_uuid"
+
+	ACTV=$(lctl get_param -n $PROC_ACT)
 	DEAC=$((1 - $ACTV))
-	set_and_check client "cat $PROC_ACT" "$PARAM" $DEAC || return 2
+	set_and_check client "lctl get_param -n $PROC_ACT" "$PARAM" $DEAC || return 2
         # also check ost_server_uuid status
-	RESULT=$(grep DEACTIV $PROC_UUID)
+	RESULT=$(lctl get_param -n $PROC_UUID | grep DEACTIV)
 	if [ -z "$RESULT" ]; then
-	    echo "Live client not deactivated: $(cat $PROC_UUID)"
+	    echo "Live client not deactivated: $(lctl get_param -n $PROC_UUID)"
 	    return 3
 	else
 	    echo "Live client success: got $RESULT"
 	fi
 
 	# check MDT too 
-	local MPROC="$LPROC/osc/$FSNAME-OST0001-osc/active"
+	local MPROC="osc.$FSNAME-OST0001-osc.active"
 	local MAX=30
 	local WAIT=0
 	while [ 1 ]; do
 	    sleep 5
-	    RESULT=`do_facet mds " [ -r $MPROC ] && cat $MPROC"`
+	    RESULT=`do_facet mds " lctl get_param -n $MPROC"`
 	    [ ${PIPESTATUS[0]} = 0 ] || error "Can't read $MPROC"
 	    if [ $RESULT -eq $DEAC ]; then
 		echo "MDT deactivated also after $WAIT sec (got $RESULT)"
@@ -1037,31 +1096,32 @@ test_29() {
         # test new client starts deactivated
  	umount_client $MOUNT || return 200
 	mount_client $MOUNT
-	RESULT=$(grep DEACTIV $PROC_UUID | grep NEW)
+	RESULT=$(lctl get_param -n $PROC_UUID | grep DEACTIV | grep NEW)
 	if [ -z "$RESULT" ]; then
-	    echo "New client not deactivated from start: $(cat $PROC_UUID)"
+	    echo "New client not deactivated from start: $(lctl get_param -n $PROC_UUID)"
 	    return 5
 	else
 	    echo "New client success: got $RESULT"
 	fi
 
 	# make sure it reactivates
-	set_and_check client "cat $PROC_ACT" "$PARAM" $ACTV || return 6
+	set_and_check client "lctl get_param -n $PROC_ACT" "$PARAM" $ACTV || return 6
 
  	umount_client $MOUNT
 	stop_ost2
 	cleanup_nocli
 	#writeconf to remove all ost2 traces for subsequent tests
 	writeconf
+	start_mds
+	start_ost
+	cleanup
 }
 run_test 29 "permanently remove an OST"
 
 test_30() {
-	# start mds first after writeconf
-	start_mds
-	start_ost
-	mount_client $MOUNT
-	TEST="cat $LPROC/llite/$FSNAME-*/max_read_ahead_whole_mb"
+	setup
+
+	TEST="lctl get_param -n llite.$FSNAME-*.max_read_ahead_whole_mb"
 	ORIG=$($TEST) 
 	for i in $(seq 1 20); do 
 	    set_and_check client "$TEST" "$FSNAME.llite.max_read_ahead_whole_mb" $i || return 3
@@ -1095,6 +1155,7 @@ test_32a() {
         [ -z "$TUNEFS" ] && skip "No tunefs" && return
 	local DISK1_4=$LUSTRE/tests/disk1_4.zip
         [ ! -r $DISK1_4 ] && skip "Cant find $DISK1_4, skipping" && return
+	mkdir -p $TMP/$tdir
 	unzip -o -j -d $TMP/$tdir $DISK1_4 || { skip "Cant unzip $DISK1_4, skipping" && return ; }
 	load_modules
 	sysctl lnet.debug=$PTLDEBUG
@@ -1102,13 +1163,13 @@ test_32a() {
 	$TUNEFS $TMP/$tdir/mds || error "tunefs failed"
 	# nids are wrong, so client wont work, but server should start
         start mds $TMP/$tdir/mds "-o loop,exclude=lustre-OST0000" || return 3
-        local UUID=$(cat $LPROC/mds/lustre-MDT0000/uuid)
+        local UUID=$(lctl get_param -n mds.lustre-MDT0000.uuid)
 	echo MDS uuid $UUID
 	[ "$UUID" == "mdsA_UUID" ] || error "UUID is wrong: $UUID" 
 
 	$TUNEFS --mgsnode=`hostname` $TMP/$tdir/ost1 || error "tunefs failed"
 	start ost1 $TMP/$tdir/ost1 "-o loop" || return 5
-        UUID=$(cat $LPROC/obdfilter/lustre-OST0000/uuid)
+	UUID=$(lctl get_param -n obdfilter.lustre-OST0000.uuid)
 	echo OST uuid $UUID
 	[ "$UUID" == "ost1_UUID" ] || error "UUID is wrong: $UUID" 
 
@@ -1123,6 +1184,16 @@ test_32a() {
 	$LCTL conf_param lustre-MDT0000.failover.node=$NID || return 10
 	echo "ok."
 
+	# With a new good MDT failover nid, we should be able to mount a client
+	# (but it cant talk to OST)
+        local OLDMOUNTOPT=$MOUNTOPT
+        MOUNTOPT="exclude=lustre-OST0000"
+	mount_client $MOUNT
+        MOUNTOPT=$OLDMOUNTOPT
+	set_and_check client "lctl get_param -n mdc.*.max_rpcs_in_flight" "lustre-MDT0000.mdc.max_rpcs_in_flight" ||
+		return 11
+
+	zconf_umount `hostname` $MOUNT -f
 	cleanup_nocli
 	load_modules
 
@@ -1133,7 +1204,7 @@ test_32a() {
         start mds $TMP/$tdir/mds "-o loop,exclude=lustre-OST0000" || return 12
         cleanup_nocli
 
-	[ -d $TMP/$tdir ] && rm -rf $TMP/$tdir
+	[ -d $TMP/$tdir ] && { rm -rf $TMP/$tdir || true; }	# true is only for TMP on NFS
 }
 run_test 32a "Upgrade from 1.4 (not live)"
 
@@ -1150,6 +1221,7 @@ test_32b() {
         [ -z "$TUNEFS" ] && skip "No tunefs" && return
 	local DISK1_4=$LUSTRE/tests/disk1_4.zip
         [ ! -r $DISK1_4 ] && skip "Cant find $DISK1_4, skipping" && return
+	mkdir -p $TMP/$tdir
 	unzip -o -j -d $TMP/$tdir $DISK1_4 || { skip "Cant unzip $DISK1_4, skipping" && return ; }
 	load_modules
 	sysctl lnet.debug=$PTLDEBUG
@@ -1158,13 +1230,13 @@ test_32b() {
 	# writeconf will cause servers to register with their current nids
 	$TUNEFS --writeconf --fsname=$NEWNAME $TMP/$tdir/mds || error "tunefs failed"
 	start mds $TMP/$tdir/mds "-o loop" || return 3
-        local UUID=$(cat $LPROC/mds/${NEWNAME}-MDT0000/uuid)
+        local UUID=$(lctl get_param -n mds.${NEWNAME}-MDT0000.uuid)
 	echo MDS uuid $UUID
 	[ "$UUID" == "mdsA_UUID" ] || error "UUID is wrong: $UUID" 
 
 	$TUNEFS --mgsnode=`hostname` --fsname=$NEWNAME --writeconf $TMP/$tdir/ost1 || error "tunefs failed"
 	start ost1 $TMP/$tdir/ost1 "-o loop" || return 5
-        UUID=$(cat $LPROC/obdfilter/${NEWNAME}-OST0000/uuid)
+        UUID=$(lctl get_param -n obdfilter.${NEWNAME}-OST0000.uuid)
 	echo OST uuid $UUID
 	[ "$UUID" == "ost1_UUID" ] || error "UUID is wrong: $UUID"
 
@@ -1183,46 +1255,47 @@ test_32b() {
 	FSNAME=$NEWNAME
 	mount_client $MOUNT
 	FSNAME=$OLDFS
-	set_and_check client "cat $LPROC/mdc/*/max_rpcs_in_flight" "${NEWNAME}-MDT0000.mdc.max_rpcs_in_flight" || return 11
+	set_and_check client "lctl get_param -n mdc.*.max_rpcs_in_flight" "${NEWNAME}-MDT0000.mdc.max_rpcs_in_flight" || return 11
 	[ "$(cksum $MOUNT/passwd | cut -d' ' -f 1,2)" == "2479747619 779" ] || return 12  
 	echo "ok."
 
 	cleanup
-	[ -d $TMP/$tdir ] && rm -rf $TMP/$tdir
+	[ -d $TMP/$tdir ] && { rm -rf $TMP/$tdir || true; }	# true is only for TMP on NFS
 }
 run_test 32b "Upgrade from 1.4 with writeconf"
 
-test_33() { # bug 12333
-        local FSNAME2=test1234
-        local fs2mds_HOST=$mds_HOST
-        local fs2ost_HOST=$ost_HOST
+test_33a() { # bug 12333, was test_33
+        local rc=0
+        local FSNAME2=test-123
+        [ -n "$ost1_HOST" ] && fs2ost_HOST=$ost1_HOST
 
-        [ -z "$fs2ost_DEV" -o -z "$fs2mds_DEV" ] && [ -b "$MDSDEV" ] && \
-            log "mixed loopback and real device not working" && return
+        if [ -z "$fs2ost_DEV" -o -z "$fs2mds_DEV" ]; then
+                do_facet mds [ -b "$MDSDEV" ] && \
+                skip "mixed loopback and real device not working" && return
+        fi
 
         local fs2mdsdev=${fs2mds_DEV:-${MDSDEV}_2}
         local fs2ostdev=${fs2ost_DEV:-$(ostdevname 1)_2}
         add fs2mds $MDS_MKFS_OPTS --fsname=${FSNAME2} --reformat $fs2mdsdev || exit 10
-        add fs2ost $OST_MKFS_OPTS --fsname=${FSNAME2} --index=8191 --mgsnode=`hostname`@tcp --reformat $fs2ostdev || exit 10
+        add fs2ost $OST_MKFS_OPTS --fsname=${FSNAME2} --index=8191 --mgsnode=$MGSNID --reformat $fs2ostdev || exit 10
 
-        start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS
+        start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS && trap cleanup_24a EXIT INT
         start fs2ost $fs2ostdev $OST_MOUNT_OPTS
+        do_facet mds "$LCTL conf_param $FSNAME2.sys.timeout=200" || rc=1
         mkdir -p $MOUNT2
-        mount -t lustre $MGSNID:/${FSNAME2} $MOUNT2 || return 1
+        mount -t lustre $MGSNID:/${FSNAME2} $MOUNT2 || rc=2
         echo "ok."
 
         umount -d $MOUNT2
         stop fs2ost -f
         stop fs2mds -f
         rm -rf $MOUNT2 $fs2mdsdev $fs2ostdev
-        cleanup_nocli || return 6
+        cleanup_nocli || rc=6
+        return $rc
 }
-run_test 33 "Mount ost with a large index number"
+run_test 33a "Mount ost with a large index number"
 
-umount_client $MOUNT	
-cleanup_nocli
-
-test_33a() {
+test_33b() {	# was test_33a
         setup
 
         do_facet client dd if=/dev/zero of=$MOUNT/24 bs=1024k count=1
@@ -1233,12 +1306,11 @@ test_33a() {
         umount_client $MOUNT
         cleanup
 }
-run_test 33a "Drop cancel during umount"
+run_test 33b "Drop cancel during umount"
 
 test_34a() {
         setup
-	do_facet client multiop $DIR/file O_c &
-	sleep 0.500s
+	do_facet client "sh runmultiop_bg_pause $DIR/file O_c"
 	manual_umount_client
 	rc=$?
 	do_facet client killall -USR1 multiop
@@ -1281,7 +1353,7 @@ test_34c() {
 	cleanup
 	return 0	
 }
-run_test 34c "force umount with failed mds should be normal"
+run_test 34c "force umount with failed ost should be normal"
 
 test_35() { # bug 12459
 	setup
@@ -1296,7 +1368,7 @@ test_35() { # bug 12459
 	log "Wait for RECONNECT_INTERVAL seconds (10s)"
 	sleep 10
 
-	local MSG="conf-sanity.sh test_33 `date +%F%kh%Mm%Ss`"
+	MSG="conf-sanity.sh test_35 `date +%F%kh%Mm%Ss`"
 	$LCTL clear
 	log "$MSG"
 	log "Stopping the MDT:"
@@ -1332,22 +1404,28 @@ run_test 35 "Reconnect to the last active server first"
 test_36() { # 12743
         local rc
         local FSNAME2=test1234
-        local fs2mds_HOST=$mds_HOST
-        local fs2ost_HOST=$ost_HOST
         local fs3ost_HOST=$ost_HOST
+
+        [ -n "$ost1_HOST" ] && fs2ost_HOST=$ost1_HOST && fs3ost_HOST=$ost1_HOST
         rc=0
 
-        [ -z "$fs2ost_DEV" -o -z "$fs2mds_DEV" ] && [ -b "$MDSDEV" ] && \
-            log "mixed loopback and real device not working" && return
-
+        if [ -z "$fs2ost_DEV" -o -z "$fs2mds_DEV" -o -z "$fs3ost_DEV" ]; then
+		do_facet mds [ -b "$MDSDEV" ] && \
+		skip "mixed loopback and real device not working" && return
+        fi
         [ $OSTCOUNT -lt 2 ] && skip "skipping test for single OST" && return
 
+	[ "$ost_HOST" = "`hostname`" -o "$ost1_HOST" = "`hostname`" ] || \
+		{ skip "remote OST" && return 0; }
+
         local fs2mdsdev=${fs2mds_DEV:-${MDSDEV}_2}
         local fs2ostdev=${fs2ost_DEV:-$(ostdevname 1)_2}
         local fs3ostdev=${fs3ost_DEV:-$(ostdevname 2)_2}
         add fs2mds $MDS_MKFS_OPTS --fsname=${FSNAME2} --reformat $fs2mdsdev || exit 10
-        add fs2ost $OST_MKFS_OPTS --mkfsoptions='-b1024' --fsname=${FSNAME2} --mgsnode=`hostname`@tcp --reformat $fs2ostdev || exit 10
-        add fs3ost $OST_MKFS_OPTS --mkfsoptions='-b4096' --fsname=${FSNAME2} --mgsnode=`hostname`@tcp --reformat $fs3ostdev || exit 10
+        # XXX after we support non 4K disk blocksize, change following --mkfsoptions with
+        # other argument
+        add fs2ost $OST_MKFS_OPTS --mkfsoptions='-b4096' --fsname=${FSNAME2} --mgsnode=$MGSNID --reformat $fs2ostdev || exit 10
+        add fs3ost $OST_MKFS_OPTS --mkfsoptions='-b4096' --fsname=${FSNAME2} --mgsnode=$MGSNID --reformat $fs3ostdev || exit 10
 
         start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS
         start fs2ost $fs2ostdev $OST_MOUNT_OPTS
@@ -1355,15 +1433,13 @@ test_36() { # 12743
         mkdir -p $MOUNT2
         mount -t lustre $MGSNID:/${FSNAME2} $MOUNT2 || return 1
 
+        sleep 5 # until 11778 fixed
+
         dd if=/dev/zero of=$MOUNT2/$tfile bs=1M count=7 || return 2
-        [ $(grep -c obdfilter $LPROC/devices) -eq 0 ] &&
-                skip "skipping test for remote OST" && return
-        BKTOTAL=`awk 'BEGIN{total=0}; {total+=$1}; END{print total}' \
-                $LPROC/obdfilter/*/kbytestotal`
-        BKFREE=`awk 'BEGIN{free=0}; {free+=$1}; END{print free}' \
-               $LPROC/obdfilter/*/kbytesfree`
-        BKAVAIL=`awk 'BEGIN{avail=0}; {avail+=$1}; END{print avail}' \
-                $LPROC/obdfilter/*/kbytesavail`
+
+        BKTOTAL=`lctl get_param -n obdfilter.*.kbytestotal | awk 'BEGIN{total=0}; {total+=$1}; END{print total}'`
+        BKFREE=`lctl get_param -n obdfilter.*.kbytesfree | awk 'BEGIN{free=0}; {free+=$1}; END{print free}'`
+        BKAVAIL=`lctl get_param -n obdfilter.*.kbytesavail | awk 'BEGIN{avail=0}; {avail+=$1}; END{print avail}'`
         STRING=`df -P $MOUNT2 | tail -n 1 | awk '{print $2","$3","$4}'`
         DFTOTAL=`echo $STRING | cut -d, -f1`
         DFUSED=`echo $STRING  | cut -d, -f2`
@@ -1398,7 +1474,34 @@ test_36() { # 12743
 }
 run_test 36 "df report consistency on OSTs with different block size"
 
-test_37() { # bug 14222
+test_37() {
+	[ -n "$CLIENTONLY" -o -n "$CLIENTMODSONLY" ] && skip "client only testing" && return 0
+	LOCAL_MDSDEV="$TMP/mdt.img"
+	SYM_MDSDEV="$TMP/sym_mdt.img"
+
+	echo "MDS :     $LOCAL_MDSDEV"
+	echo "SYMLINK : $SYM_MDSDEV"
+	rm -f $LOCAL_MDSDEV
+
+	touch $LOCAL_MDSDEV
+	mkfs.lustre --reformat --fsname=lustre --mdt --mgs --device-size=9000 $LOCAL_MDSDEV ||
+		error "mkfs.lustre $LOCAL_MDSDEV failed"
+	ln -s $LOCAL_MDSDEV $SYM_MDSDEV
+
+	echo "mount symlink device - $SYM_MDSDEV"
+
+	mount_op=`mount -v -t lustre -o loop $SYM_MDSDEV ${MOUNT%/*}/mds 2>&1 | grep "unable to set tunable"`
+	umount -d ${MOUNT%/*}/mds
+	rm -f $LOCAL_MDSDEV $SYM_MDSDEV
+
+	if [ -n "$mount_op" ]; then
+		error "**** FAIL: set tunables failed for symlink device"
+	fi
+	return 0
+}
+run_test 37 "verify set tunables works for symlink device"
+
+test_38() { # bug 14222
 	setup
 	# like runtests
 	COUNT=10
@@ -1413,80 +1516,64 @@ test_37() { # bug 14222
 	stop_mds
 	log "rename lov_objid file on MDS"
 	rm -f $TMP/lov_objid.orig
-	do_facet mds "debugfs -w $MDSDEV" <<-EOF
-		dump lov_objid $TMP/lov_objid.orig
-		rm lov_objid
-	EOF
+	do_facet mds "debugfs -c -R \\\"dump lov_objid $TMP/lov_objid.orig\\\" $MDSDEV"
+	do_facet mds "debugfs -w -R \\\"rm lov_objid\\\" $MDSDEV"
+
 	do_facet mds "od -Ax -td8 $TMP/lov_objid.orig"
 	# check create in mds_lov_connect
 	start_mds
 	mount_client $MOUNT
 	for f in $FILES; do
 		[ $V ] && log "verifying $DIR/$tdir/$f"
-		diff $f $DIR/$tdir/$f || ERROR=y
+		diff -q $f $DIR/$tdir/$f || ERROR=y
 	done
-	do_facet mds "debugfs -c $MDSDEV" <<-EOF
-		dump lov_objid $TMP/lov_objid.new
-	EOF
+	do_facet mds "debugfs -c -R \\\"dump lov_objid $TMP/lov_objid.new\\\"  $MDSDEV"
 	do_facet mds "od -Ax -td8 $TMP/lov_objid.new"
 	[ "$ERROR" = "y" ] && error "old and new files are different after connect" || true
 	
 	
 	# check it's updates in sync
 	umount_client $MOUNT
-	stop_mds	
-	multiop $TMP/lov_objid.clear Ow4096c
-	do_facet mds "debugfs -w $MDSDEV" <<-EOF
-		write $TMP/lov_objid.clear lov_objid
-	EOF
+	stop_mds
+	
+	do_facet mds dd if=/dev/zero of=$TMP/lov_objid.clear bs=4096 count=1
+	do_facet mds "debugfs -w -R \\\"rm lov_objid\\\" $MDSDEV"
+	do_facet mds "debugfs -w -R \\\"write $TMP/lov_objid.clear lov_objid\\\" $MDSDEV "
+
 	start_mds
 	mount_client $MOUNT
 	for f in $FILES; do
 		[ $V ] && log "verifying $DIR/$tdir/$f"
-		diff $f $DIR/$tdir/$f || ERROR=y
+		diff -q $f $DIR/$tdir/$f || ERROR=y
 	done
-	do_facet mds "debugfs -c $MDSDEV" <<-EOF
-		dump lov_objid $TMP/lov_objid.new1
-	EOF
+        do_facet mds "debugfs -c -R \\\"dump lov_objid $TMP/lov_objid.new1\\\" $MDSDEV"
 	do_facet mds "od -Ax -td8 $TMP/lov_objid.new1"
 	umount_client $MOUNT
 	stop_mds
 	[ "$ERROR" = "y" ] && error "old and new files are different after sync" || true
 	
 	log "files compared the same"
-	#cleanup
-}
-run_test 37 "MDS recreates missing lov_objid file from OST data"
-
-test_39() { # bug 14631
-	setup
-	# like runtests
-	COUNT=10
-	SRC="/etc /bin"
-	FILES=`find $SRC -type f -mtime +1 | head -n $COUNT`
-	log "copying $(echo $FILES | wc -w) files to $DIR/$tdir"
-	mkdir -p $DIR/$tdir
-	tar cf - $FILES | tar xf - -C $DIR/$tdir || \
-		error "copying $SRC to $DIR/$tdir"
-	sync
-	grep " $MOUNT " /proc/mounts && zconf_umount `hostname` $MOUNT $*
-	for num in `seq $OSTCOUNT`; do
-		stop ost$num -f
-	done
-	stop mds -f
-	start_mds
-	start_ost
-	mount_client $MOUNT
-	log "comparing previously copied files"
-	for f in $FILES; do
-		[ $V ] && log "verifying $DST/$f"
-		diff -q $f $DST/$f || ERROR=22
-	done
-	[ "$ERROR" ] && error "file comparison failed"
 	cleanup
 }
-run_test 39 "Check valid del orphan"
+run_test 38 "MDS recreates missing lov_objid file from OST data"
 
+test_39() { #bug 14413
+        PTLDEBUG=+malloc
+        setup
+        cleanup
+        perl $SRCDIR/leak_finder.pl $TMP/debug 2>&1 | egrep '*** Leak:' && 
+                error "memory leak detected" || true
+}
+run_test 39 "leak_finder recognizes both LUSTRE and LNET malloc messages"
+
+test_40() { # bug 15759
+	start_ost
+	#define OBD_FAIL_TGT_TOOMANY_THREADS     0x706
+	do_facet mds "sysctl -w lustre.fail_loc=0x80000706"
+	start_mds
+	cleanup
+}
+run_test 40 "race during service thread startup"
 
 equals_msg `basename $0`: test complete
 [ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG || true
