@@ -1,7 +1,7 @@
 #!/bin/bash
 # vim:expandtab:shiftwidth=4:softtabstop=4:tabstop=4:
 
-trap 'echo "test-framework exiting on error"' ERR
+trap 'print_summary && echo "test-framework exiting on error"' ERR
 set -e
 #set -x
 
@@ -31,6 +31,38 @@ usage() {
     exit
 }
 
+print_summary () {
+    [ "$TESTSUITE" == "lfscktest" ] && return 0
+    [ -n "$ONLY" ] && echo "WARNING: ONLY is set to ${ONLY}."
+    local form="%-13s %-17s %s\n"
+    printf "$form" "status" "script" "skipped tests E(xcluded) S(low)"
+    echo "------------------------------------------------------------------------------------"
+    for O in $TESTSUITE_LIST; do
+        local skipped=""
+        local slow=""
+        local o=$(echo $O | tr "[:upper:]" "[:lower:]")
+        o=${o//_/-}
+        o=${o//tyn/tyN}
+        local log=${TMP}/${o}.log 
+        [ -f $log ] && skipped=$(grep excluded $log | awk '{ printf " %s", $3 }' | sed 's/test_//g')
+        [ -f $log ] && slow=$(grep SLOW $log | awk '{ printf " %s", $3 }' | sed 's/test_//g')
+        [ "${!O}" = "done" ] && \
+            printf "$form" "Done" "$O" "E=$skipped" && \
+            [ -n "$slow" ] && printf "$form" "-" "-" "S=$slow"
+
+    done
+
+    for O in $TESTSUITE_LIST; do
+        [ "${!O}" = "no" ] && \
+            printf "$form" "Skipped" "$O" ""
+    done
+
+    for O in $TESTSUITE_LIST; do
+        [ "${!O}" = "done" -o "${!O}" = "no" ] || \
+            printf "$form" "UNFINISHED" "$O" ""
+    done
+}
+
 init_test_env() {
     export LUSTRE=`absolute_path $LUSTRE`
     export TESTSUITE=`basename $0 .sh`
@@ -39,6 +71,7 @@ init_test_env() {
     [ -d /r ] && export ROOT=${ROOT:-/r}
     export TMP=${TMP:-$ROOT/tmp}
     export TESTSUITELOG=${TMP}/${TESTSUITE}.log
+    export HOSTNAME=${HOSTNAME:-`hostname`}
 
     export PATH=:$PATH:$LUSTRE/utils:$LUSTRE/tests
     export LCTL=${LCTL:-"$LUSTRE/utils/lctl"}
@@ -55,6 +88,7 @@ init_test_env() {
     export NAME=${NAME:-local}
     export LPROC=/proc/fs/lustre
     export DIR2
+    export AT_MAX_PATH
 
     if [ "$ACCEPTOR_PORT" ]; then
         export PORT_OPT="--port $ACCEPTOR_PORT"
@@ -114,6 +148,8 @@ load_modules() {
 
     echo Loading modules from $LUSTRE
     load_module ../lnet/libcfs/libcfs
+    [ "$PTLDEBUG" ] && sysctl -w lnet.debug=$PTLDEBUG
+    [ "$SUBSYSTEM" ] && sysctl -w lnet.subsystem_debug=${SUBSYSTEM# }
     [ -f /etc/modprobe.conf ] && MODPROBECONF=/etc/modprobe.conf
     [ -f /etc/modprobe.d/Lustre ] && MODPROBECONF=/etc/modprobe.d/Lustre
     [ -z "$LNETOPTS" -a -n "$MODPROBECONF" ] && \
@@ -131,9 +167,10 @@ load_modules() {
     load_module osc/osc
     load_module lov/lov
     load_module mgc/mgc
-    if [ -z "$CLIENTONLY" ]; then
+    if [ -z "$CLIENTONLY" ] && [ -z "$CLIENTMODSONLY" ]; then
         load_module mgs/mgs
         load_module mds/mds
+        grep -q crc16 /proc/kallsyms || { modprobe crc16 2>/dev/null || true; }
         [ "$FSTYPE" = "ldiskfs" ] && load_module ../ldiskfs/ldiskfs/ldiskfs
         load_module lvfs/fsfilt_$FSTYPE
         load_module ost/ost
@@ -142,10 +179,10 @@ load_modules() {
 
     load_module llite/lustre
     load_module llite/llite_lloop
-    rm -f $TMP/ogdb-`hostname`
+    rm -f $TMP/ogdb-$HOSTNAME
     OGDB=$TMP
     [ -d /r ] && OGDB="/r/tmp"
-    $LCTL modules > $OGDB/ogdb-`hostname`
+    $LCTL modules > $OGDB/ogdb-$HOSTNAME
     # 'mount' doesn't look in $PATH, just sbin
     [ -f $LUSTRE/utils/mount.lustre ] && cp $LUSTRE/utils/mount.lustre /sbin/. || true
 }
@@ -181,25 +218,27 @@ wait_for_lnet() {
 unload_modules() {
     wait_exit_ST client # bug 12845
 
-    lsmod | grep lnet > /dev/null && $LCTL dl && $LCTL dk $TMP/debug
-    local MODULES=$($LCTL modules | awk '{ print $2 }')
+    lsmod | grep libcfs > /dev/null && $LCTL dl
+    local MODULES=$($LCTL modules | awk '{ print $2 }' | grep -v libcfs) || true
     $RMMOD $MODULES > /dev/null 2>&1 || true
      # do it again, in case we tried to unload ksocklnd too early
-    MODULES=$($LCTL modules | awk '{ print $2 }')
+    MODULES=$($LCTL modules | awk '{ print $2 }' | grep -v libcfs) || true
     [ -n "$MODULES" ] && $RMMOD $MODULES > /dev/null 2>&1 || true
+    lsmod | grep libcfs > /dev/null && $LCTL dk $TMP/debug
+    $RMMOD libcfs
     MODULES=$($LCTL modules | awk '{ print $2 }')
     if [ -n "$MODULES" ]; then
-    echo "Modules still loaded: "
-    echo $MODULES 
-    if [ -e $LPROC ]; then
-        echo "Lustre still loaded"
-        cat $LPROC/devices || true
-        lsmod
-        return 2
-    else
-        echo "Lustre stopped but LNET is still loaded, waiting..."
-        wait_for_lnet || return 3
-    fi
+        echo "Modules still loaded: "
+        echo $MODULES 
+        if [ "$(lctl dl)" ]; then
+            echo "Lustre still loaded"
+            lctl dl || true
+            lsmod
+            return 2
+        else
+            echo "Lustre stopped but LNET is still loaded, waiting..."
+            wait_for_lnet || return 3
+        fi
     fi
     HAVE_MODULES=false
 
@@ -210,7 +249,7 @@ unload_modules() {
         echo "$LEAK_PORTALS" 1>&2
         mv $TMP/debug $TMP/debug-leak.`date +%s` || true
         echo "Memory leaks detected"
-	[ -n "$IGNORE_LEAK" ] && echo "ignoring leaks" && return 0
+        [ -n "$IGNORE_LEAK" ] && echo "ignoring leaks" && return 0
         return 254
     fi
     echo "modules unloaded."
@@ -218,30 +257,41 @@ unload_modules() {
 }
 
 # Facet functions
-# start facet device options 
-start() {
-    facet=$1
+mount_facet() {
+    local facet=$1
     shift
-    device=$1
-    shift
-    echo "Starting ${facet}: $@ ${device} ${MOUNT%/*}/${facet}"
-    do_facet ${facet} mkdir -p ${MOUNT%/*}/${facet}
-    do_facet ${facet} mount -t lustre $@ ${device} ${MOUNT%/*}/${facet} 
+    local dev=${facet}_dev
+    local opt=${facet}_opt
+    echo "Starting ${facet}: ${!opt} $@ ${!dev} ${MOUNT%/*}/${facet}"
+    do_facet ${facet} mount -t lustre ${!opt} $@ ${!dev} ${MOUNT%/*}/${facet}     
     RC=${PIPESTATUS[0]}
     if [ $RC -ne 0 ]; then
         echo "mount -t lustre $@ ${device} ${MOUNT%/*}/${facet}" 
         echo "Start of ${device} on ${facet} failed ${RC}"
     else 
         do_facet ${facet} "sysctl -w lnet.debug=$PTLDEBUG; \
-        sysctl -w lnet.subsystem_debug=${SUBSYSTEM# }; \
-        sysctl -w lnet.debug_mb=${DEBUG_SIZE}"
+            sysctl -w lnet.subsystem_debug=${SUBSYSTEM# }; \
+            sysctl -w lnet.debug_mb=${DEBUG_SIZE}; \
+            sync"
+    fi
+    return $RC
+}
 
-        do_facet ${facet} sync
+# start facet device options 
+start() {
+    facet=$1
+    shift
+    device=$1
+    shift
+    eval export ${facet}_dev=${device}
+    eval export ${facet}_opt=\"$@\"
+    do_facet ${facet} mkdir -p ${MOUNT%/*}/${facet}
+    mount_facet ${facet}
+    RC=$?
+    if [ $RC -eq 0 ]; then
         label=$(do_facet ${facet} "e2label ${device}")
         [ -z "$label" ] && echo no label for ${device} && exit 1
         eval export ${facet}_svc=${label}
-        eval export ${facet}_dev=${device}
-        eval export ${facet}_opt=\"$@\"
         echo Started ${label}
     fi
     return $RC
@@ -287,7 +337,7 @@ zconf_mount() {
     do_node $client "sysctl -w lnet.debug=$PTLDEBUG;
         sysctl -w lnet.subsystem_debug=${SUBSYSTEM# };
         sysctl -w lnet.debug_mb=${DEBUG_SIZE}"
-    [ -d /r ] && $LCTL modules > /r/tmp/ogdb-`hostname`
+    [ -d /r ] && $LCTL modules > /r/tmp/ogdb-$HOSTNAME
     return 0
 }
 
@@ -298,6 +348,7 @@ zconf_umount() {
     local running=$(do_node $client "grep -c $mnt' ' /proc/mounts") || true
     if [ $running -ne 0 ]; then
         echo "Stopping client $mnt (opts:$force)"
+        lsof | grep "$mnt" || true
         do_node $client umount $force $mnt
     fi
 }
@@ -383,7 +434,7 @@ wait_for() {
 }
 
 wait_mds_recovery_done () {
-    local timeout=`do_facet mds cat /proc/sys/lustre/timeout`
+    local timeout=`do_facet mds sysctl -n lustre.timeout`
 #define OBD_RECOVERY_TIMEOUT (obd_timeout * 5 / 2)
 # as we are in process of changing obd_timeout in different ways
 # let's set MAX longer than that
@@ -407,7 +458,7 @@ wait_exit_ST () {
     local INTERVAL=1
     # conf-sanity 31 takes a long time cleanup
     while [ $WAIT -lt 300 ]; do
-        running=$(do_facet ${facet} "[ -e $LPROC ] && grep ST' ' $LPROC/devices") || true
+        running=$(do_facet ${facet} "lsmod | grep lnet > /dev/null && lctl dl | grep ' ST '") || true
         [ -z "${running}" ] && return 0
         echo "waited $WAIT for${running}"
         [ $INTERVAL -lt 64 ] && INTERVAL=$((INTERVAL + INTERVAL))
@@ -451,9 +502,7 @@ facet_failover() {
     TO=`facet_active_host $facet`
     echo "Failover $facet to $TO"
     wait_for $facet
-    local dev=${facet}_dev
-    local opt=${facet}_opt
-    start $facet ${!dev} ${!opt} || error "Restart of $facet failed"
+    mount_facet $facet || error "Restart of $facet failed"
 }
 
 obd_name() {
@@ -497,15 +546,16 @@ fail() {
     df $MOUNT || error "post-failover df: $?"
 }
 
+fail_nodf() {
+    local facet=$1
+    facet_failover $facet
+}
+
 fail_abort() {
     local facet=$1
     stop $facet
     change_active $facet
-    local svc=${facet}_svc
-    local dev=${facet}_dev
-    local opt=${facet}_opt
-    start $facet ${!dev} ${!opt}
-    do_facet $facet lctl --device %${!svc} abort_recovery
+    mount_facet $facet -o abort_recovery
     df $MOUNT || echo "first df failed: $?"
     sleep 1
     df $MOUNT || error "post-failover df: $?"
@@ -593,7 +643,7 @@ facet_active_host() {
     local facet=$1
     local active=`facet_active $facet`
     if [ "$facet" == client ]; then
-        hostname
+        echo $HOSTNAME
     else
         echo `facet_host $active`
     fi
@@ -619,13 +669,27 @@ do_node() {
     HOST=$1
     shift
     local myPDSH=$PDSH
-    if [ "$HOST" = "$(hostname)" ]; then
+    if [ "$HOST" = "$HOSTNAME" ]; then
         myPDSH="no_dsh"
+    elif [ -z "$myPDSH" -o "$myPDSH" = "no_dsh" ]; then
+        echo "cannot run remote command on $HOST with $myPDSH"
+        return 128
     fi
     if $VERBOSE; then
         echo "CMD: $HOST $@" >&2
         $myPDSH $HOST $LCTL mark "$@" > /dev/null 2>&1 || :
     fi
+
+    if [ "$myPDSH" = "rsh" ]; then
+# we need this because rsh does not return exit code of an executed command
+	local command_status="$TMP/cs"
+	rsh $HOST ":> $command_status"
+	rsh $HOST "(PATH=\$PATH:$RLUSTRE/utils:$RLUSTRE/tests:/sbin:/usr/sbin;
+		    cd $RPWD; sh -c \"$@\") || 
+		    echo command failed >$command_status"
+	[ -n "$($myPDSH $HOST cat $command_status)" ] && return 1 || true
+        return 0
+    fi
     $myPDSH $HOST "(PATH=\$PATH:$RLUSTRE/utils:$RLUSTRE/tests:/sbin:/usr/sbin; cd $RPWD; sh -c \"$@\")" | sed "s/^${HOST}: //"
     return ${PIPESTATUS[0]}
 }
@@ -667,8 +731,8 @@ stopall() {
     fi
     
     # assume client mount is local 
-    grep " $MOUNT " /proc/mounts && zconf_umount `hostname` $MOUNT $*
-    grep " $MOUNT2 " /proc/mounts && zconf_umount `hostname` $MOUNT2 $*
+    grep " $MOUNT " /proc/mounts && zconf_umount $HOSTNAME $MOUNT $*
+    grep " $MOUNT2 " /proc/mounts && zconf_umount $HOSTNAME $MOUNT2 $*
     [ "$CLIENTONLY" ] && return
     stop mds -f
     for num in `seq $OSTCOUNT`; do
@@ -706,7 +770,23 @@ formatall() {
 }
 
 mount_client() {
-    grep " $1 " /proc/mounts || zconf_mount `hostname` $*
+    grep " $1 " /proc/mounts || zconf_mount $HOSTNAME $*
+}
+
+remount_client()
+{
+	zconf_umount `hostname` $1 || error "umount failed"
+	zconf_mount `hostname` $1 || error "mount failed"
+}
+
+set_obd_timeout() {
+    local facet=$1
+    local timeout=$2
+
+    do_facet $facet lsmod | grep -q obdclass || \
+        do_facet $facet "modprobe obdclass"
+
+    do_facet $facet "sysctl -w lustre.timeout=$timeout"
 }
 
 setupall() {
@@ -715,9 +795,11 @@ setupall() {
         echo Setup mdt, osts
         echo $REFORMAT | grep -q "reformat" \
 	    || do_facet mds "$TUNEFS --writeconf $MDSDEV"
+        set_obd_timeout mds $TIMEOUT
         start mds $MDSDEV $MDS_MOUNT_OPTS
         for num in `seq $OSTCOUNT`; do
             DEVNAME=`ostdevname $num`
+            set_obd_timeout ost$num $TIMEOUT
             start ost$num $DEVNAME $OST_MOUNT_OPTS
         done
     fi
@@ -760,7 +842,7 @@ cleanup_and_setup_lustre() {
 
 check_and_cleanup_lustre() {
     if [ "`mount | grep $MOUNT`" ]; then
-        rm -rf $DIR/[Rdfs][1-9]*
+        [ -n "$DIR" ] && rm -rf $DIR/[Rdfs][0-9]*
     fi
     if [ "$I_MOUNTED" = "yes" ]; then
         cleanupall -f || error "cleanup failed"
@@ -811,6 +893,52 @@ absolute_path() {
     (cd `dirname $1`; echo $PWD/`basename $1`)
 }
 
+##################################
+# Adaptive Timeouts funcs
+
+at_is_valid() {
+    if [ -z "$AT_MAX_PATH" ]; then
+        AT_MAX_PATH=$(do_facet mds "find /sys/ -name at_max")
+        [ -z "$AT_MAX_PATH" ] && echo "missing /sys/.../at_max " && return 1
+    fi
+    return 0
+}
+
+at_is_enabled() {
+    at_is_valid || error "invalid call"
+
+    # only check mds, we assume at_max is the same on all nodes
+    local at_max=$(do_facet mds "cat $AT_MAX_PATH")
+    if [ $at_max -eq 0 ]; then
+        return 1
+    else
+        return 0
+    fi
+}
+
+at_max_get() {
+    at_is_valid || error "invalid call"
+
+    do_facet $1 "cat $AT_MAX_PATH"
+}
+
+at_max_set() {
+    local at_max=$1
+    shift
+
+    at_is_valid || error "invalid call"
+
+    for facet in $@; do
+        if [ $facet == "ost" ]; then
+            for i in `seq $OSTCOUNT`; do
+                do_facet ost$i "echo $at_max > $AT_MAX_PATH"
+            done
+        else
+            do_facet $facet "echo $at_max > $AT_MAX_PATH"
+        fi
+    done
+}
+
 ##################################
 # OBD_FAIL funcs
 
@@ -883,7 +1011,7 @@ clear_failloc() {
     pause=$2
     sleep $pause
     echo "clearing fail_loc on $facet"
-    do_facet $facet "sysctl -w lustre.fail_loc=0"
+    do_facet $facet "sysctl -e -w lustre.fail_loc=0"
 }
 
 cancel_lru_locks() {
@@ -895,6 +1023,35 @@ cancel_lru_locks() {
     $LCTL mark "cancel_lru_locks $1 stop"
 }
 
+default_lru_size()
+{
+        NR_CPU=$(grep -c "processor" /proc/cpuinfo)
+        DEFAULT_LRU_SIZE=$((100 * NR_CPU))
+        echo "$DEFAULT_LRU_SIZE"
+}
+
+lru_resize_enable()
+{
+        NS=$1
+        test "x$NS" = "x" && NS="mdc"
+        for F in $LPROC/ldlm/namespaces/*$NS*/lru_size; do
+                D=$(dirname $F)
+                log "Enable lru resize for $(basename $D)"
+                echo "0" > $F
+        done
+}
+
+lru_resize_disable()
+{
+        NS=$1
+        test "x$NS" = "x" && NS="mdc"
+        for F in $LPROC/ldlm/namespaces/*$NS*/lru_size; do
+                D=$(dirname $F)
+                log "Disable lru resize for $(basename $D)"
+                DEFAULT_LRU_SIZE=$(default_lru_size)
+                echo "$DEFAULT_LRU_SIZE" > $F
+        done
+}
 
 pgcache_empty() {
     for a in /proc/fs/lustre/llite/*/dump_page_cache; do
@@ -920,11 +1077,10 @@ debugrestore() {
 # Test interface 
 ##################################
 
-error() {
-    local FAIL_ON_ERROR=${FAIL_ON_ERROR:-true}
+error_noexit() {
     local TYPE=${TYPE:-"FAIL"}
     local ERRLOG
-    sysctl -w lustre.fail_loc=0 2> /dev/null || true
+    sysctl -e -w lustre.fail_loc=0 || true
     log " ${TESTSUITE} ${TESTNAME}: @@@@@@ ${TYPE}: $@ "
     ERRLOG=$TMP/lustre_${TESTSUITE}_${TESTNAME}.$(date +%s)
     echo "Dumping lctl log to $ERRLOG"
@@ -935,16 +1091,25 @@ error() {
     done
     debugrestore
     [ "$TESTSUITELOG" ] && echo "$0: ${TYPE}: $TESTNAME $@" >> $TESTSUITELOG
-    if $FAIL_ON_ERROR; then
-	exit 1
-    fi
+}
+
+error() {
+    error_noexit "$@"
+    $FAIL_ON_ERROR && exit 1 || true
+}
+
+error_exit() {
+    error_noexit "$@"
+    exit 1
 }
 
 # use only if we are ignoring failures for this test, bugno required.
 # (like ALWAYS_EXCEPT, but run the test and ignore the results.)
 # e.g. error_ignore 5494 "your message"
 error_ignore() {
-    FAIL_ON_ERROR=false TYPE="IGNORE (bz$1)" error $2
+    TYPE="IGNORE (bz$1)"
+    shift
+    error_noexit "$@"
 }
 
 skip () {
@@ -959,9 +1124,14 @@ build_test_filter() {
     done
     [ "$EXCEPT$ALWAYS_EXCEPT" ] && \
         log "skipping tests: `echo $EXCEPT $ALWAYS_EXCEPT`"
+    [ "$EXCEPT_SLOW" ] && \
+        log "skipping tests SLOW=no: `echo $EXCEPT_SLOW`"
     for E in $EXCEPT $ALWAYS_EXCEPT; do
         eval EXCEPT_${E}=true
     done
+    for E in $EXCEPT_SLOW; do
+        eval EXCEPT_SLOW_${E}=true
+    done
     for G in $GRANT_CHECK_LIST; do
         eval GCHECK_ONLY_${G}=true
    	done
@@ -1001,6 +1171,17 @@ run_test() {
         TESTNAME=test_$1 skip "skipping excluded test $1 (base $base)"
         return 0
     fi
+    testname=EXCEPT_SLOW_$1
+    if [ ${!testname}x != x ]; then
+        TESTNAME=test_$1 skip "skipping SLOW test $1"
+        return 0
+    fi
+    testname=EXCEPT_SLOW_$base
+    if [ ${!testname}x != x ]; then
+        TESTNAME=test_$1 skip "skipping SLOW test $1 (base $base)"
+        return 0
+    fi
+
     run_one $1 "$2"
     
     return $?
@@ -1026,6 +1207,8 @@ log() {
     MSG=${MSG//\)/\\\)}
     MSG=${MSG//\;/\\\;}
     MSG=${MSG//\|/\\\|}
+    MSG=${MSG//\>/\\\>}
+    MSG=${MSG//\</\\\<}
     local NODES=$(nodes_list)
     for NODE in $NODES; do
         do_node $NODE $LCTL mark "$MSG" 2> /dev/null || true
@@ -1050,11 +1233,23 @@ check_mds() {
     [ $FFREE -ge $FTOTAL ] && error "files free $FFREE > total $FTOTAL" || true
 }
 
+reset_fail_loc () {
+    local myNODES=$(nodes_list)
+    local NODE
+
+    for NODE in $myNODES; do
+        do_node $NODE sysctl -e -w lustre.fail_loc=0 || true
+    done
+}
+
 run_one() {
     testnum=$1
     message=$2
     tfile=f${testnum}
-    export tdir=d${base}
+    export tdir=d0.${TESTSUITE}/d${base}
+    local SAVE_UMASK=`umask`
+    umask 0022
+    mkdir -p $DIR/$tdir
 
     BEFORE=`date +%s`
     log "== test $testnum: $message ============ `date +%H:%M:%S` ($BEFORE)"
@@ -1062,12 +1257,16 @@ run_one() {
     export TESTNAME=test_$testnum
     test_${testnum} || error "test_$testnum failed with $?"
     #check_mds
+    reset_fail_loc
     check_grant ${testnum} || error "check_grant $testnum failed with $?"
     [ -f $CATASTROPHE ] && [ `cat $CATASTROPHE` -ne 0 ] && \
         error "LBUG/LASSERT detected"
+    ps auxww | grep -v grep | grep -q multiop && error "multiop still running"
     pass "($((`date +%s` - $BEFORE))s)"
+    rmdir ${DIR}/$tdir >/dev/null 2>&1 || true
     unset TESTNAME
     unset tdir
+    umask $SAVE_UMASK
     cd $SAVE_PWD
     $CLEANUP
 }
@@ -1094,7 +1293,7 @@ check_grant() {
 	# write some data to sync client lost_grant
 	rm -f $DIR1/${tfile}_check_grant_* 2>&1
 	for i in `seq $OSTCOUNT`; do
-		$LFS setstripe $DIR1/${tfile}_check_grant_$i 0 $(($i -1)) 1
+		$LFS setstripe $DIR1/${tfile}_check_grant_$i -i $(($i -1)) -c 1
 		dd if=/dev/zero of=$DIR1/${tfile}_check_grant_$i bs=4k \
 					      count=1 > /dev/null 2>&1 
 	done
@@ -1140,38 +1339,107 @@ osc_to_ost()
 
 remote_mds ()
 {
-    [ ! -e /proc/fs/lustre/mds/*MDT* ]
+    [ -z "$(lctl dl | grep mdt)" ]
+}
+
+remote_mds_nodsh()
+{
+    remote_mds && [ "$PDSH" = "no_dsh" -o -z "$PDSH" -o -z "$mds_HOST" ]
 }
 
 remote_ost ()
 {
-    [ $(grep -c obdfilter $LPROC/devices) -eq 0 ]
+    [ -z "$(lctl dl | grep ost)" ]
+}
+
+remote_ost_nodsh()
+{
+    remote_ost && [ "$PDSH" = "no_dsh" -o -z "$PDSH" -o -z "$ost_HOST" ]
 }
 
 osts_nodes () {
     local OSTNODES=$(facet_host ost1)
+    local NODES_sort
 
     for num in `seq $OSTCOUNT`; do
         local myOST=$(facet_host ost$num)
-        [[ ! '\ '"$OSTNODES"'\ ' = *'\ '"$myOST"'\ '* ]] && OSTNODES="$OSTNODES $myOST"
+        OSTNODES="$OSTNODES $myOST"
     done
+    NODES_sort=$(for i in $OSTNODES; do echo $i; done | sort -u)
 
-    echo $OSTNODES
+    echo $NODES_sort
 }
 
 nodes_list () {
     # FIXME. We need a list of clients
-    local myNODES=`hostname`
+    local myNODES=$HOSTNAME
+    local myNODES_sort
 
-    [ ! "$mds_HOST" = "$(hostname)" ] && myNODES="$myNODES $mds_HOST"
+    if [ "$PDSH" -a "$PDSH" != "no_dsh" ]; then
+        myNODES="$myNODES $(osts_nodes) $mds_HOST"
+    fi
 
-    local OSTNODES=$(osts_nodes)
-    local myOSTNODES=`echo ' '"$OSTNODES"' ' | sed -e s/[\ ]$mds_HOST[\ ]/\ / -e s/[\ ]$(hostname)[\ ]/\ /`
-    [ -n "$myOSTNODES" ] && myNODES="$myNODES $myOSTNODES"
-    echo $myNODES
+    myNODES_sort=$(for i in $myNODES; do echo $i; done | sort -u)
+
+    echo $myNODES_sort
 }
 
 is_patchless ()
 {
-    grep -q patchless $LPROC/version
+    lctl get_param version | grep -q patchless
+}
+
+check_runas_id_ret() {
+    local myRC=0
+    local myRUNAS_ID=$1
+    shift
+    local myRUNAS=$@
+    if [ -z "$myRUNAS" ]; then
+        error_exit "myRUNAS command must be specified for check_runas_id"
+    fi
+    mkdir $DIR/d0_runas_test
+    chmod 0755 $DIR
+    chown $myRUNAS_ID:$myRUNAS_ID $DIR/d0_runas_test
+    $myRUNAS touch $DIR/d0_runas_test/f$$ || myRC=1
+    rm -rf $DIR/d0_runas_test
+    return $myRC
+}
+
+check_runas_id() {
+    local myRUNAS_ID=$1
+    shift
+    local myRUNAS=$@
+    check_runas_id_ret $myRUNAS_ID $myRUNAS || \
+        error "unable to write to $DIR/d0_runas_test as UID $myRUNAS_ID. 
+        Please set RUNAS_ID to some UID which exists on MDS and client or 
+        add user $myRUNAS_ID:$myRUNAS_ID on these nodes."
+}
+
+# Run multiop in the background, but wait for it to print
+# "PAUSING" to its stdout before returning from this function.
+multiop_bg_pause() {
+    MULTIOP_PROG=${MULTIOP_PROG:-multiop}
+    FILE=$1
+    ARGS=$2
+
+    TMPPIPE=/tmp/multiop_open_wait_pipe.$$
+    mkfifo $TMPPIPE
+
+    echo "$MULTIOP_PROG $FILE v$ARGS"
+    $MULTIOP_PROG $FILE v$ARGS > $TMPPIPE &
+
+    echo "TMPPIPE=${TMPPIPE}"
+    read -t 60 multiop_output < $TMPPIPE
+    if [ $? -ne 0 ]; then
+        rm -f $TMPPIPE
+        return 1
+    fi
+    rm -f $TMPPIPE
+    if [ "$multiop_output" != "PAUSING" ]; then
+        echo "Incorrect multiop output: $multiop_output"
+        kill -9 $PID
+        return 1
+    fi
+
+    return 0
 }
